# pylint: disable=invalid-name,no-self-use,protected-access

from __future__ import absolute_import
from numpy.testing import assert_almost_equal
import torch

from allennlp.common import Params
from allennlp.common.testing import AllenNlpTestCase
from allennlp.models.semantic_parsing.wikitables.wikitables_decoder_state import WikiTablesDecoderState
from allennlp.models.semantic_parsing.wikitables.wikitables_decoder_step import WikiTablesDecoderStep
from allennlp.modules import SimilarityFunction
from allennlp.nn.decoding import GrammarState, RnnState
from allennlp.semparse.type_declarations.type_declaration import is_nonterminal


class WikiTablesDecoderStepTest(AllenNlpTestCase):
    def setUp(self):
        super(WikiTablesDecoderStepTest, self).setUp()

        batch_indices = [0, 1, 0]
        action_history = [[1], [3, 4], []]
        score = [torch.FloatTensor([x]) for x in [.1, 1.1, 2.2]]
        hidden_state = torch.FloatTensor([[i, i] for i in range(len(batch_indices))])
        memory_cell = torch.FloatTensor([[i, i] for i in range(len(batch_indices))])
        previous_action_embedding = torch.FloatTensor([[i, i] for i in range(len(batch_indices))])
        attended_question = torch.FloatTensor([[i, i] for i in range(len(batch_indices))])
        grammar_state = [GrammarState([u'e'], {}, {}, {}, is_nonterminal) for _ in batch_indices]
        self.encoder_outputs = torch.FloatTensor([[1, 2], [3, 4], [5, 6]])
        self.encoder_output_mask = torch.FloatTensor([[1, 1], [1, 0], [1, 1]])
        self.action_embeddings = torch.FloatTensor([[0, 0], [1, 1], [2, 2], [3, 3], [4, 4], [5, 5]])
        self.output_action_embeddings = torch.FloatTensor([[0, 0], [1, 1], [2, 2], [3, 3], [4, 4], [5, 5]])
        self.action_biases = torch.FloatTensor([[0], [1], [2], [3], [4], [5]])
        self.action_indices = {
                (0, 0): 1,
                (0, 1): 0,
                (0, 2): 2,
                (0, 3): 3,
                (0, 4): 5,
                (1, 0): 4,
                (1, 1): 0,
                (1, 2): 2,
                (1, 3): 3,
                }
        self.possible_actions = [[(u'e -> ', False, None),
                                  (u'e -> g', True, None),
                                  (u'e -> h', True, None),
                                  (u'e -> i', True, None),
                                  (u'e -> j', True, None)],
                                 [(u'e -> q', True, None),
                                  (u'e -> g', True, None),
                                  (u'e -> h', True, None),
                                  (u'e -> i', True, None)]]

        # (batch_size, num_entities, num_question_tokens) = (2, 5, 3)
        linking_scores = torch.Tensor([[[.1, .2, .3],
                                        [.4, .5, .6],
                                        [.7, .8, .9],
                                        [1.0, 1.1, 1.2],
                                        [1.3, 1.4, 1.5]],
                                       [[-.1, -.2, -.3],
                                        [-.4, -.5, -.6],
                                        [-.7, -.8, -.9],
                                        [-1.0, -1.1, -1.2],
                                        [-1.3, -1.4, -1.5]]])
        flattened_linking_scores = linking_scores.view(2 * 5, 3)

        # Maps (batch_index, action_index) to indices into the flattened linking score tensor,
        # which has shae (batch_size * num_entities, num_question_tokens).
        actions_to_entities = {
                (0, 0): 0,
                (0, 1): 1,
                (0, 2): 2,
                (0, 6): 3,
                (1, 3): 6,
                (1, 4): 7,
                (1, 5): 8,
                }
        entity_types = {
                0: 0,
                1: 2,
                2: 1,
                3: 0,
                4: 0,
                5: 1,
                6: 0,
                7: 1,
                8: 2,
                }
        rnn_state = []
        for i in range(len(batch_indices)):
            rnn_state.append(RnnState(hidden_state[i],
                                      memory_cell[i],
                                      previous_action_embedding[i],
                                      attended_question[i],
                                      self.encoder_outputs,
                                      self.encoder_output_mask))
        self.state = WikiTablesDecoderState(batch_indices=batch_indices,
                                            action_history=action_history,
                                            score=score,
                                            rnn_state=rnn_state,
                                            grammar_state=grammar_state,
                                            action_embeddings=self.action_embeddings,
                                            output_action_embeddings=self.output_action_embeddings,
                                            action_biases=self.action_biases,
                                            action_indices=self.action_indices,
                                            possible_actions=self.possible_actions,
                                            flattened_linking_scores=flattened_linking_scores,
                                            actions_to_entities=actions_to_entities,
                                            entity_types=entity_types)

    def test_get_actions_to_consider(self):
        # pylint: disable=protected-access
        valid_actions_1 = {u'e': [0, 1, 2, 4]}
        valid_actions_2 = {u'e': [0, 1, 3]}
        valid_actions_3 = {u'e': [2, 3, 4]}
        self.state.grammar_state[0] = GrammarState([u'e'], {}, valid_actions_1, {}, is_nonterminal)
        self.state.grammar_state[1] = GrammarState([u'e'], {}, valid_actions_2, {}, is_nonterminal)
        self.state.grammar_state[2] = GrammarState([u'e'], {}, valid_actions_3, {}, is_nonterminal)

        # We're making a bunch of the actions linked actions here, pretending that there are only
        # two global actions.
        self.state.action_indices = {
                (0, 0): 1,
                (0, 1): 0,
                (0, 2): -1,
                (0, 3): -1,
                (0, 4): -1,
                (1, 0): -1,
                (1, 1): 0,
                (1, 2): -1,
                (1, 3): -1,
                }

        considered, to_embed, to_link = WikiTablesDecoderStep._get_actions_to_consider(self.state)
        # These are _global_ action indices.  They come from actions [[(0, 0), (0, 1)], [(1, 1)], []].
        expected_to_embed = [[1, 0], [0], []]
        assert to_embed == expected_to_embed
        # These are _batch_ action indices with a _global_ action index of -1.
        # They come from actions [[(0, 2), (0, 4)], [(1, 0), (1, 3)], [(0, 2), (0, 3), (0, 4)]].
        expected_to_link = [[2, 4], [0, 3], [2, 3, 4]]
        assert to_link == expected_to_link
        # These are _batch_ action indices, with padding in between the embedded actions and the
        # linked actions (and after the linked actions, if necessary).
        expected_considered = [[0, 1, 2, 4, -1], [1, -1, 0, 3, -1], [-1, -1, 2, 3, 4]]
        assert considered == expected_considered

    def test_get_actions_to_consider_returns_none_if_no_linked_actions(self):
        # pylint: disable=protected-access
        valid_actions_1 = {u'e': [0, 1, 2, 4]}
        valid_actions_2 = {u'e': [0, 1, 3]}
        valid_actions_3 = {u'e': [2, 3, 4]}
        self.state.grammar_state[0] = GrammarState([u'e'], {}, valid_actions_1, {}, is_nonterminal)
        self.state.grammar_state[1] = GrammarState([u'e'], {}, valid_actions_2, {}, is_nonterminal)
        self.state.grammar_state[2] = GrammarState([u'e'], {}, valid_actions_3, {}, is_nonterminal)
        considered, to_embed, to_link = WikiTablesDecoderStep._get_actions_to_consider(self.state)
        # These are _global_ action indices.  All of the actions in this case are embedded, so this
        # is just a mapping from the valid actions above to their global ids.
        expected_to_embed = [[1, 0, 2, 5], [4, 0, 3], [2, 3, 5]]
        assert to_embed == expected_to_embed
        # There are no linked actions (all of them are embedded), so this should be None.
        assert to_link is None
        # These are _batch_ action indices, with padding in between the embedded actions and the
        # linked actions.  Because there are no linked actions, this is basically just the
        # valid_actions for each group element padded with -1s.
        expected_considered = [[0, 1, 2, 4], [0, 1, 3, -1], [2, 3, 4, -1]]
        assert considered == expected_considered

    def test_get_action_embeddings(self):
        action_embeddings = torch.rand(5, 4)
        self.state.action_embeddings = action_embeddings
        self.state.output_action_embeddings = action_embeddings
        self.state.action_biases = torch.rand(5, 1)
        actions_to_embed = [[0, 4], [1], [2, 3, 4]]
        embeddings, _, _, mask = WikiTablesDecoderStep._get_action_embeddings(self.state, actions_to_embed)
        assert_almost_equal(mask.detach().cpu().numpy(), [[1, 1, 0], [1, 0, 0], [1, 1, 1]])
        assert tuple(embeddings.size()) == (3, 3, 4)
        assert_almost_equal(embeddings[0, 0].detach().cpu().numpy(), action_embeddings[0].detach().cpu().numpy())
        assert_almost_equal(embeddings[0, 1].detach().cpu().numpy(), action_embeddings[4].detach().cpu().numpy())
        assert_almost_equal(embeddings[0, 2].detach().cpu().numpy(), action_embeddings[0].detach().cpu().numpy())
        assert_almost_equal(embeddings[1, 0].detach().cpu().numpy(), action_embeddings[1].detach().cpu().numpy())
        assert_almost_equal(embeddings[1, 1].detach().cpu().numpy(), action_embeddings[0].detach().cpu().numpy())
        assert_almost_equal(embeddings[1, 2].detach().cpu().numpy(), action_embeddings[0].detach().cpu().numpy())
        assert_almost_equal(embeddings[2, 0].detach().cpu().numpy(), action_embeddings[2].detach().cpu().numpy())
        assert_almost_equal(embeddings[2, 1].detach().cpu().numpy(), action_embeddings[3].detach().cpu().numpy())
        assert_almost_equal(embeddings[2, 2].detach().cpu().numpy(), action_embeddings[4].detach().cpu().numpy())

    def test_get_entity_action_logits(self):
        decoder_step = WikiTablesDecoderStep(1, 5, SimilarityFunction.from_params(Params({})), 5, 3)
        actions_to_link = [[1, 2], [3, 4, 5], [6]]
        # (group_size, num_question_tokens) = (3, 3)
        attention_weights = torch.Tensor([[.2, .8, 0],
                                          [.7, .1, .2],
                                          [.3, .3, .4]])
        action_logits, mask, type_embeddings = decoder_step._get_entity_action_logits(self.state,
                                                                                      actions_to_link,
                                                                                      attention_weights)
        assert_almost_equal(mask.detach().cpu().numpy(), [[1, 1, 0], [1, 1, 1], [1, 0, 0]])

        assert tuple(action_logits.size()) == (3, 3)
        assert_almost_equal(action_logits[0, 0].detach().cpu().numpy(), .4 * .2 + .5 * .8 + .6 * 0)
        assert_almost_equal(action_logits[0, 1].detach().cpu().numpy(), .7 * .2 + .8 * .8 + .9 * 0)
        assert_almost_equal(action_logits[1, 0].detach().cpu().numpy(), -.4 * .7 + -.5 * .1 + -.6 * .2)
        assert_almost_equal(action_logits[1, 1].detach().cpu().numpy(), -.7 * .7 + -.8 * .1 + -.9 * .2)
        assert_almost_equal(action_logits[1, 2].detach().cpu().numpy(), -1.0 * .7 + -1.1 * .1 + -1.2 * .2)
        assert_almost_equal(action_logits[2, 0].detach().cpu().numpy(), 1.0 * .3 + 1.1 * .3 + 1.2 * .4)

        embedding_matrix = decoder_step._entity_type_embedding.weight.detach().cpu().numpy()
        assert_almost_equal(type_embeddings[0, 0].detach().cpu().numpy(), embedding_matrix[2])
        assert_almost_equal(type_embeddings[0, 1].detach().cpu().numpy(), embedding_matrix[1])
        assert_almost_equal(type_embeddings[1, 0].detach().cpu().numpy(), embedding_matrix[0])
        assert_almost_equal(type_embeddings[1, 1].detach().cpu().numpy(), embedding_matrix[1])
        assert_almost_equal(type_embeddings[1, 2].detach().cpu().numpy(), embedding_matrix[2])
        assert_almost_equal(type_embeddings[2, 0].detach().cpu().numpy(), embedding_matrix[0])

    def test_compute_new_states(self):
        # pylint: disable=protected-access
        log_probs = torch.FloatTensor([[.1, .9, -.1, .2],
                                       [.3, 1.1, .1, .8],
                                       [.1, .25, .3, .4]])
        considered_actions = [[0, 1, 2, 3], [0, -1, 3, -1], [0, 2, 4, -1]]
        allowed_actions = [set([2, 3]), set([0]), set([4])]
        max_actions = 1
        step_action_embeddings = torch.FloatTensor([[[1, 1], [9, 9], [2, 2], [3, 3]],
                                                    [[4, 4], [9, 9], [3, 3], [9, 9]],
                                                    [[1, 1], [2, 2], [5, 5], [9, 9]]])
        new_hidden_state = torch.FloatTensor([[i + 1, i + 1] for i in range(len(allowed_actions))])
        new_memory_cell = torch.FloatTensor([[i + 1, i + 1] for i in range(len(allowed_actions))])
        new_attended_question = torch.FloatTensor([[i + 1, i + 1] for i in range(len(allowed_actions))])
        new_attention_weights = torch.FloatTensor([[i + 1, i + 1] for i in range(len(allowed_actions))])
        new_states = WikiTablesDecoderStep._compute_new_states(self.state,
                                                               log_probs,
                                                               new_hidden_state,
                                                               new_memory_cell,
                                                               step_action_embeddings,
                                                               new_attended_question,
                                                               new_attention_weights,
                                                               considered_actions,
                                                               allowed_actions,
                                                               max_actions)

        assert len(new_states) == 2
        new_state = new_states[0]
        # For batch instance 0, we should have selected action 4 from group index 2.
        assert new_state.batch_indices == [0]
        assert_almost_equal(new_state.score[0].detach().cpu().numpy().tolist(), [.3])
        # These have values taken from what's defined in setUp() - the prior action history
        # (empty in this case)  and the nonterminals corresponding to the action we picked ('j').
        assert new_state.action_history == [[4]]
        assert new_state.grammar_state[0]._nonterminal_stack == [u'j']
        # All of these values come from the objects instantiated directly above.
        assert_almost_equal(new_state.rnn_state[0].hidden_state.cpu().numpy().tolist(), [3, 3])
        assert_almost_equal(new_state.rnn_state[0].memory_cell.cpu().numpy().tolist(), [3, 3])
        assert_almost_equal(new_state.rnn_state[0].previous_action_embedding.cpu().numpy().tolist(), [5, 5])
        assert_almost_equal(new_state.rnn_state[0].attended_input.cpu().numpy().tolist(), [3, 3])
        # And these should just be copied from the prior state.
        assert_almost_equal(new_state.rnn_state[0].encoder_outputs.cpu().numpy(),
                            self.encoder_outputs.cpu().numpy())
        assert_almost_equal(new_state.rnn_state[0].encoder_output_mask.detach().cpu().numpy(),
                            self.encoder_output_mask.detach().cpu().numpy())
        assert_almost_equal(new_state.action_embeddings.cpu().numpy(),
                            self.action_embeddings.cpu().numpy())
        assert new_state.action_indices == self.action_indices
        assert new_state.possible_actions == self.possible_actions

        new_state = new_states[1]
        # For batch instance 1, we should have selected action 0 from group index 1.
        assert new_state.batch_indices == [1]
        assert_almost_equal(new_state.score[0].detach().cpu().numpy().tolist(), [.3])
        # These two have values taken from what's defined in setUp() - the prior action history
        # ([3, 4]) and the nonterminals corresponding to the action we picked ('q').
        assert new_state.action_history == [[3, 4, 0]]
        assert new_state.grammar_state[0]._nonterminal_stack == [u'q']
        # All of these values come from the objects instantiated directly above.
        assert_almost_equal(new_state.rnn_state[0].hidden_state.cpu().numpy().tolist(), [2, 2])
        assert_almost_equal(new_state.rnn_state[0].memory_cell.cpu().numpy().tolist(), [2, 2])
        assert_almost_equal(new_state.rnn_state[0].previous_action_embedding.cpu().numpy().tolist(), [4, 4])
        assert_almost_equal(new_state.rnn_state[0].attended_input.cpu().numpy().tolist(), [2, 2])
        # And these should just be copied from the prior state.
        assert_almost_equal(new_state.rnn_state[0].encoder_outputs.cpu().numpy(),
                            self.encoder_outputs.cpu().numpy())
        assert_almost_equal(new_state.rnn_state[0].encoder_output_mask.detach().cpu().numpy(),
                            self.encoder_output_mask.detach().cpu().numpy())
        assert_almost_equal(new_state.action_embeddings.cpu().numpy(),
                            self.action_embeddings.cpu().numpy())
        assert new_state.action_indices == self.action_indices
        assert new_state.possible_actions == self.possible_actions

    def test_compute_new_states_with_no_action_constraints(self):
        # pylint: disable=protected-access
        # This test is basically identical to the previous one, but without specifying
        # `allowed_actions`.  This makes sure we get the right behavior at test time.
        log_probs = torch.FloatTensor([[.1, .9, -.1, .2],
                                       [.3, 1.1, .1, .8],
                                       [.1, .25, .3, .4]])
        considered_actions = [[0, 1, 2, 3], [0, -1, 3, -1], [0, 2, 4, -1]]
        max_actions = 1
        step_action_embeddings = torch.FloatTensor([[[1, 1], [9, 9], [2, 2], [3, 3]],
                                                    [[4, 4], [9, 9], [3, 3], [9, 9]],
                                                    [[1, 1], [2, 2], [5, 5], [9, 9]]])
        new_hidden_state = torch.FloatTensor([[i + 1, i + 1] for i in range(len(considered_actions))])
        new_memory_cell = torch.FloatTensor([[i + 1, i + 1] for i in range(len(considered_actions))])
        new_attended_question = torch.FloatTensor([[i + 1, i + 1] for i in range(len(considered_actions))])
        new_attention_weights = torch.FloatTensor([[i + 1, i + 1] for i in range(len(considered_actions))])
        new_states = WikiTablesDecoderStep._compute_new_states(self.state,
                                                               log_probs,
                                                               new_hidden_state,
                                                               new_memory_cell,
                                                               step_action_embeddings,
                                                               new_attended_question,
                                                               new_attention_weights,
                                                               considered_actions,
                                                               allowed_actions=None,
                                                               max_actions=max_actions)

        assert len(new_states) == 2
        new_state = new_states[0]
        # For batch instance 0, we should have selected action 1 from group index 0.
        assert new_state.batch_indices == [0]
        assert_almost_equal(new_state.score[0].detach().cpu().numpy().tolist(), [.9])
        # These two have values taken from what's defined in setUp() - the prior action history
        # ([1]) and the nonterminals corresponding to the action we picked ('j').
        assert new_state.action_history == [[1, 1]]
        assert new_state.grammar_state[0]._nonterminal_stack == [u'g']
        # All of these values come from the objects instantiated directly above.
        assert_almost_equal(new_state.rnn_state[0].hidden_state.cpu().numpy().tolist(), [1, 1])
        assert_almost_equal(new_state.rnn_state[0].memory_cell.cpu().numpy().tolist(), [1, 1])
        assert_almost_equal(new_state.rnn_state[0].previous_action_embedding.cpu().numpy().tolist(), [9, 9])
        assert_almost_equal(new_state.rnn_state[0].attended_input.cpu().numpy().tolist(), [1, 1])
        # And these should just be copied from the prior state.
        assert_almost_equal(new_state.rnn_state[0].encoder_outputs.cpu().numpy(),
                            self.encoder_outputs.cpu().numpy())
        assert_almost_equal(new_state.rnn_state[0].encoder_output_mask.detach().cpu().numpy(),
                            self.encoder_output_mask.detach().cpu().numpy())
        assert_almost_equal(new_state.action_embeddings.cpu().numpy(),
                            self.action_embeddings.cpu().numpy())
        assert new_state.action_indices == self.action_indices
        assert new_state.possible_actions == self.possible_actions

        new_state = new_states[1]
        # For batch instance 0, we should have selected action 0 from group index 1.
        assert new_state.batch_indices == [1]
        assert_almost_equal(new_state.score[0].detach().cpu().numpy().tolist(), [.3])
        # These have values taken from what's defined in setUp() - the prior action history
        # ([3, 4]) and the nonterminals corresponding to the action we picked ('q').
        assert new_state.action_history == [[3, 4, 0]]
        assert new_state.grammar_state[0]._nonterminal_stack == [u'q']
        # All of these values come from the objects instantiated directly above.
        assert_almost_equal(new_state.rnn_state[0].hidden_state.cpu().numpy().tolist(), [2, 2])
        assert_almost_equal(new_state.rnn_state[0].memory_cell.cpu().numpy().tolist(), [2, 2])
        assert_almost_equal(new_state.rnn_state[0].previous_action_embedding.cpu().numpy().tolist(), [4, 4])
        assert_almost_equal(new_state.rnn_state[0].attended_input.cpu().numpy().tolist(), [2, 2])
        # And these should just be copied from the prior state.
        assert_almost_equal(new_state.rnn_state[0].encoder_outputs.cpu().numpy(),
                            self.encoder_outputs.cpu().numpy())
        assert_almost_equal(new_state.rnn_state[0].encoder_output_mask.detach().cpu().numpy(),
                            self.encoder_output_mask.detach().cpu().numpy())
        assert_almost_equal(new_state.action_embeddings.cpu().numpy(),
                            self.action_embeddings.cpu().numpy())
        assert new_state.action_indices == self.action_indices
        assert new_state.possible_actions == self.possible_actions




from __future__ import absolute_import
#typing

#overrides
import torch
from torch.nn import Conv1d, Linear

from allennlp.modules.seq2vec_encoders.seq2vec_encoder import Seq2VecEncoder
from allennlp.nn import Activation


class CnnEncoder(Seq2VecEncoder):
 
    def __init__(self,
                 embedding_dim     ,
                 num_filters     ,
                 ngram_filter_sizes                  = (2, 3, 4, 5),  # pylint: disable=bad-whitespace
                 conv_layer_activation             = None,
                 output_dim                = None)        :
        super(CnnEncoder, self).__init__()
        self._embedding_dim = embedding_dim
        self._num_filters = num_filters
        self._ngram_filter_sizes = ngram_filter_sizes
        self._activation = conv_layer_activation or Activation.by_name(u'relu')()
        self._output_dim = output_dim

        self._convolution_layers = [Conv1d(in_channels=self._embedding_dim,
                                           out_channels=self._num_filters,
                                           kernel_size=ngram_size)
                                    for ngram_size in self._ngram_filter_sizes]
        for i, conv_layer in enumerate(self._convolution_layers):
            self.add_module(u'conv_layer_%d' % i, conv_layer)

        maxpool_output_dim = self._num_filters * len(self._ngram_filter_sizes)
        if self._output_dim:
            self.projection_layer = Linear(maxpool_output_dim, self._output_dim)
        else:
            self.projection_layer = None
            self._output_dim = maxpool_output_dim

    #overrides
    def get_input_dim(self)       :
        return self._embedding_dim

    #overrides
    def get_output_dim(self)       :
        return self._output_dim

    def forward(self, tokens              , mask              ):  # pylint: disable=arguments-differ
        if mask is not None:
            tokens = tokens * mask.unsqueeze(-1).float()

        # Our input is expected to have shape `(batch_size, num_tokens, embedding_dim)`.  The
        # convolution layers expect input of shape `(batch_size, in_channels, sequence_length)`,
        # where the conv layer `in_channels` is our `embedding_dim`.  We thus need to transpose the
        # tensor first.
        tokens = torch.transpose(tokens, 1, 2)
        # Each convolution layer returns output of size `(batch_size, num_filters, pool_length)`,
        # where `pool_length = num_tokens - ngram_size + 1`.  We then do an activation function,
        # then do max pooling over each filter for the whole input sequence.  Because our max
        # pooling is simple, we just use `torch.max`.  The resultant tensor of has shape
        # `(batch_size, num_conv_layers * num_filters)`, which then gets projected using the
        # projection layer, if requested.

        filter_outputs = []
        for i in range(len(self._convolution_layers)):
            convolution_layer = getattr(self, u'conv_layer_{}'.format(i))
            filter_outputs.append(
                    self._activation(convolution_layer(tokens)).max(dim=2)[0]
            )

        # Now we have a list of `num_conv_layers` tensors of shape `(batch_size, num_filters)`.
        # Concatenating them gives us a tensor of shape `(batch_size, num_filters * num_conv_layers)`.
        maxpool_output = torch.cat(filter_outputs, dim=1) if len(filter_outputs) > 1 else filter_outputs[0]

        if self.projection_layer:
            result = self.projection_layer(maxpool_output)
        else:
            result = maxpool_output
        return result

CnnEncoder = Seq2VecEncoder.register(u"cnn")(CnnEncoder)



# pylint: disable=no-self-use,invalid-name


from __future__ import division
from __future__ import absolute_import
#typing

import pytest

from allennlp.data.dataset_readers import WinobiasReader
from allennlp.common.util import ensure_list
from allennlp.common.testing import AllenNlpTestCase
try:
    from itertools import izip
except:
    izip = zip


class TestWinobiasReader(object):
    span_width = 5

    @pytest.mark.parametrize(u"lazy", (True, False))
    def test_read_from_file(self, lazy):
        conll_reader = WinobiasReader(max_span_width=self.span_width, lazy=lazy)
        instances = ensure_list(conll_reader.read(unicode(AllenNlpTestCase.FIXTURES_ROOT /
                                                      u'coref' / u'winobias.sample')))

        assert len(instances) == 2

        fields = instances[0].fields
        text = [x.text for x in fields[u"text"].tokens]
        assert text == [u'The', u'designer', u'argued', u'with', u'the', u'developer',
                        u'and', u'slapped', u'her', u'in', u'the', u'face', u'.']

        spans = fields[u"spans"].field_list
        span_starts, span_ends = izip(*[(field.span_start, field.span_end) for field in spans])

        candidate_mentions = self.check_candidate_mentions_are_well_defined(span_starts, span_ends, text)

        gold_span_labels = fields[u"span_labels"]
        gold_indices_with_ids = [(i, x) for i, x in enumerate(gold_span_labels.labels) if x != -1]
        gold_mentions_with_ids                              = [(candidate_mentions[i], x)
                                                               for i, x in gold_indices_with_ids]
        assert gold_mentions_with_ids == [([u'the', u'developer'], 0), ([u'her'], 0)]

        fields = instances[1].fields
        text = [x.text for x in fields[u"text"].tokens]
        assert text == [u'The', u'salesperson', u'sold', u'some', u'books', u'to', u'the',
                        u'librarian', u'because', u'she', u'was', u'trying', u'to', u'sell', u'them', u'.']

        spans = fields[u"spans"].field_list
        span_starts, span_ends = izip(*[(field.span_start, field.span_end) for field in spans])
        candidate_mentions = self.check_candidate_mentions_are_well_defined(span_starts, span_ends, text)

        gold_span_labels = fields[u"span_labels"]
        gold_indices_with_ids = [(i, x) for i, x in enumerate(gold_span_labels.labels) if x != -1]
        gold_mentions_with_ids                              = [(candidate_mentions[i], x)
                                                               for i, x in gold_indices_with_ids]
        assert gold_mentions_with_ids == [([u'The', u'salesperson'], 0),
                                          ([u'some', u'books'], 1),
                                          ([u'she'], 0), ([u'them'], 1)]

    def check_candidate_mentions_are_well_defined(self, span_starts, span_ends, text):
        candidate_mentions = []
        for start, end in izip(span_starts, span_ends):
            # Spans are inclusive.
            text_span = text[start:(end + 1)]
            candidate_mentions.append(text_span)

        # Check we aren't considering zero length spans and all
        # candidate spans are less than what we specified
        assert all([self.span_width >= len(x) > 0 for x in candidate_mentions])  # pylint: disable=len-as-condition
        return candidate_mentions



u"""
We define a simple deterministic decoder here, that takes steps to add integers to list. At
each step, the decoder takes the last integer in the list, and adds either 1 or 2 to produce the
next element that will be added to the list. We initialize the list with the value 0 (or whatever
you pick), and we say that a sequence is finished when the last element is 4. We define the score
of a state as the negative of the number of elements (excluding the initial value) in the action
history.
"""

from __future__ import absolute_import
from collections import defaultdict
#typing

#overrides
import torch

from allennlp.nn.decoding import DecoderState, DecoderStep
try:
    from itertools import izip
except:
    izip = zip


class SimpleDecoderState(DecoderState[u'SimpleDecoderState']):
    def __init__(self,
                 batch_indices           ,
                 action_history                 ,
                 score                    ,
                 start_values            = None)        :
        super(SimpleDecoderState, self).__init__(batch_indices, action_history, score)
        self.start_values = start_values or [0] * len(batch_indices)

    def is_finished(self)        :
        return self.action_history[0][-1] == 4

    @classmethod
    def combine_states(cls, states)                        :
        batch_indices = [batch_index for state in states for batch_index in state.batch_indices]
        action_histories = [action_history for state in states for action_history in
                            state.action_history]
        scores = [score for state in states for score in state.score]
        start_values = [start_value for state in states for start_value in state.start_values]
        return SimpleDecoderState(batch_indices, action_histories, scores, start_values)

    def __repr__(self):
        return "{self.action_history}"


class SimpleDecoderStep(DecoderStep[SimpleDecoderState]):
    def __init__(self,
                 valid_actions           = None,
                 include_value_in_score       = False)        :
        # The default allowed actions are adding 1 or 2 to the last element.
        self._valid_actions = valid_actions or set([1, 2])
        # If True, we will add a small multiple of the action take to the score, to encourage
        # getting higher numbers first (and to differentiate action sequences).
        self._include_value_in_score = include_value_in_score

    #overrides
    def take_step(self,
                  state                    ,
                  max_actions      = None,
                  allowed_actions            = None)                            :
        indexed_next_states                                      = defaultdict(list)
        if not allowed_actions:
            allowed_actions = [None] * len(state.batch_indices)
        for batch_index, action_history, score, start_value, actions in izip(state.batch_indices,
                                                                            state.action_history,
                                                                            state.score,
                                                                            state.start_values,
                                                                            allowed_actions):

            prev_action = action_history[-1] if action_history else start_value
            for action in self._valid_actions:
                next_item = int(prev_action + action)
                if actions and next_item not in actions:
                    continue
                new_history = action_history + [next_item]
                # For every action taken, we reduce the score by 1.
                new_score = score - 1
                if self._include_value_in_score:
                    new_score += 0.01 * next_item
                new_state = SimpleDecoderState([batch_index],
                                               [new_history],
                                               [new_score])
                indexed_next_states[batch_index].append(new_state)
        next_states                           = []
        for batch_next_states in list(indexed_next_states.values()):
            sorted_next_states = [(-state.score[0].data[0], state) for state in batch_next_states]
            sorted_next_states.sort(key=lambda x: x[0])
            if max_actions is not None:
                sorted_next_states = sorted_next_states[:max_actions]
            next_states.extend(state[1] for state in sorted_next_states)
        return next_states



# pylint: disable=invalid-name


from __future__ import division
from __future__ import absolute_import
import numpy
import torch

from allennlp.common.testing import ModelTestCase
from allennlp.nn.util import sequence_cross_entropy_with_logits


class SimpleSeq2SeqWithoutAttentionTest(ModelTestCase):
    def setUp(self):
        super(SimpleSeq2SeqWithoutAttentionTest, self).setUp()
        self.set_up_model(self.FIXTURES_ROOT / u"encoder_decoder" / u"simple_seq2seq" / u"experiment.json",
                          self.FIXTURES_ROOT / u"data" / u"seq2seq_copy.tsv")

    def test_encoder_decoder_can_train_save_and_load(self):
        self.ensure_model_can_train_save_and_load(self.param_file)

    def test_loss_is_computed_correctly(self):
        batch_size = 5
        num_decoding_steps = 5
        num_classes = 10
        sample_logits = torch.randn(batch_size, num_decoding_steps-1, num_classes)
        sample_targets = torch.from_numpy(numpy.random.randint(0, num_classes,
                                                               (batch_size, num_decoding_steps)))
        # Mask should be either 0 or 1
        sample_mask = torch.from_numpy(numpy.random.randint(0, 2,
                                                            (batch_size, num_decoding_steps)))
        expected_loss = sequence_cross_entropy_with_logits(sample_logits, sample_targets[:, 1:].contiguous(),
                                                           sample_mask[:, 1:].contiguous())
        # pylint: disable=protected-access
        actual_loss = self.model._get_loss(sample_logits, sample_targets, sample_mask)
        assert numpy.equal(expected_loss.data.numpy(), actual_loss.data.numpy())

    def test_decode_runs_correctly(self):
        training_tensors = self.dataset.as_tensor_dict()
        output_dict = self.model(**training_tensors)
        decode_output_dict = self.model.decode(output_dict)
        # ``decode`` should have added a ``predicted_tokens`` field to ``output_dict``. Checking if it's there.
        assert u"predicted_tokens" in decode_output_dict

class SimpleSeq2SeqWithAttentionTest(ModelTestCase):
    def setUp(self):
        super(SimpleSeq2SeqWithAttentionTest, self).setUp()
        self.set_up_model(self.FIXTURES_ROOT / u"encoder_decoder" /
                          u"simple_seq2seq" / u"experiment_with_attention.json",
                          self.FIXTURES_ROOT / u"data" / u"seq2seq_copy.tsv")

    def test_encoder_decoder_can_train_save_and_load(self):
        self.ensure_model_can_train_save_and_load(self.param_file)



# pylint: disable=invalid-name


from __future__ import division
from __future__ import absolute_import
from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Vocabulary
from allennlp.common.params import Params
from allennlp.models.simple_tagger import SimpleTagger
from allennlp.data.dataset_readers import SequenceTaggingDatasetReader
from allennlp.training.optimizers import Optimizer
from allennlp.training.trainer import Trainer
from allennlp.data.iterators import BasicIterator


class TestOptimizer(AllenNlpTestCase):
    def setUp(self):
        super(TestOptimizer, self).setUp()
        self.instances = SequenceTaggingDatasetReader().read(self.FIXTURES_ROOT / u'data' / u'sequence_tagging.tsv')
        vocab = Vocabulary.from_instances(self.instances)
        self.model_params = Params({
                u"text_field_embedder": {
                        u"tokens": {
                                u"type": u"embedding",
                                u"embedding_dim": 5
                                }
                        },
                u"encoder": {
                        u"type": u"lstm",
                        u"input_size": 5,
                        u"hidden_size": 7,
                        u"num_layers": 2
                        }
                })
        self.model = SimpleTagger.from_params(vocab=vocab, params=self.model_params)

    def test_optimizer_basic(self):
        optimizer_params = Params({
                u"type": u"sgd",
                u"lr": 1
        })
        parameters = [[n, p] for n, p in self.model.named_parameters() if p.requires_grad]
        optimizer = Optimizer.from_params(parameters, optimizer_params)
        param_groups = optimizer.param_groups
        assert len(param_groups) == 1
        assert param_groups[0][u'lr'] == 1

    def test_optimizer_parameter_groups(self):
        optimizer_params = Params({
                u"type": u"sgd",
                u"lr": 1,
                u"momentum": 5,
                u"parameter_groups": [
                        # the repeated "bias_" checks a corner case
                        # NOT_A_VARIABLE_NAME displays a warning but does not raise an exception
                        [[u"weight_i", u"bias_", u"bias_", u"NOT_A_VARIABLE_NAME"], {u'lr': 2}],
                        [[u"tag_projection_layer"], {u'lr': 3}],
                ]
        })
        parameters = [[n, p] for n, p in self.model.named_parameters() if p.requires_grad]
        optimizer = Optimizer.from_params(parameters, optimizer_params)
        param_groups = optimizer.param_groups

        assert len(param_groups) == 3
        assert param_groups[0][u'lr'] == 2
        assert param_groups[1][u'lr'] == 3
        # base case uses default lr
        assert param_groups[2][u'lr'] == 1
        for k in range(3):
            assert param_groups[k][u'momentum'] == 5

        # all LSTM parameters except recurrent connections (those with weight_h in name)
        assert len(param_groups[0][u'params']) == 6
        # just the projection weight and bias
        assert len(param_groups[1][u'params']) == 2
        # the embedding + recurrent connections left in the default group
        assert len(param_groups[2][u'params']) == 3


class TestDenseSparseAdam(AllenNlpTestCase):

    def setUp(self):
        super(TestDenseSparseAdam, self).setUp()
        self.instances = SequenceTaggingDatasetReader().read(self.FIXTURES_ROOT / u'data' / u'sequence_tagging.tsv')
        self.vocab = Vocabulary.from_instances(self.instances)
        self.model_params = Params({
                u"text_field_embedder": {
                        u"tokens": {
                                u"type": u"embedding",
                                u"embedding_dim": 5,
                                u"sparse": True
                                }
                        },
                u"encoder": {
                        u"type": u"lstm",
                        u"input_size": 5,
                        u"hidden_size": 7,
                        u"num_layers": 2
                        }
                })
        self.model = SimpleTagger.from_params(vocab=self.vocab, params=self.model_params)

    def test_can_optimise_model_with_dense_and_sparse_params(self):
        optimizer_params = Params({
                u"type": u"dense_sparse_adam"
        })
        parameters = [[n, p] for n, p in self.model.named_parameters() if p.requires_grad]
        optimizer = Optimizer.from_params(parameters, optimizer_params)
        iterator = BasicIterator(2)
        iterator.index_with(self.vocab)
        Trainer(self.model, optimizer, iterator, self.instances).train()



import random
import time
import unittest

try:
    range = xrange
except NameError: # pragma: NO COVER  (Python3)
    pass


class UnboundedCacheTests(unittest.TestCase):

    def _getTargetClass(self):
        from repoze.lru import UnboundedCache
        return UnboundedCache

    def _makeOne(self):
        return self._getTargetClass()()

    def test_ctor(self):
        cache = self._makeOne()
        self.assertEqual(cache._data, {})

    def test_get_miss_no_default(self):
        cache = self._makeOne()
        self.assertIsNone(cache.get('nonesuch'))

    def test_get_miss_explicit_default(self):
        cache = self._makeOne()
        default = object()
        self.assertIs(cache.get('nonesuch', default), default)

    def test_get_hit(self):
        cache = self._makeOne()
        extant = cache._data['extant'] = object()
        self.assertIs(cache.get('extant'), extant)

    def test_clear(self):
        cache = self._makeOne()
        extant = cache._data['extant'] = object()
        cache.clear()
        self.assertIsNone(cache.get('extant'))

    def test_invalidate_miss(self):
        cache = self._makeOne()
        cache.invalidate('nonesuch')  # does not raise

    def test_invalidate_hit(self):
        cache = self._makeOne()
        extant = cache._data['extant'] = object()
        cache.invalidate('extant')
        self.assertIsNone(cache.get('extant'))

    def test_put(self):
        cache = self._makeOne()
        extant = object()
        cache.put('extant', extant)
        self.assertIs(cache._data['extant'], extant)


class LRUCacheTests(unittest.TestCase):

    def _getTargetClass(self):
        from repoze.lru import LRUCache
        return LRUCache

    def _makeOne(self, size):
        return self._getTargetClass()(size)

    def check_cache_is_consistent(self, cache):
        #Return if cache is consistent, else raise fail test case.
        # cache.hand/maxpos/size
        self.assertTrue(cache.hand < len(cache.clock_keys))
        self.assertTrue(cache.hand >= 0)
        self.assertEqual(cache.maxpos, cache.size - 1)
        self.assertEqual(len(cache.clock_keys), cache.size)

        # lengths of data structures
        self.assertEqual(len(cache.clock_keys), len(cache.clock_refs))
        self.assertTrue(len(cache.data) <= len(cache.clock_refs))

        # For each item in cache.data
        #   1. pos must be a valid index
        #   2. clock_keys must point back to the entry
        for key, value in cache.data.items():
            pos, val = value
            self.assertTrue(
                    type(pos) == type(42) or
                    type(pos) == type(2 ** 128))
            self.assertTrue(pos >= 0)
            self.assertTrue(pos <= cache.maxpos)

            clock_key = cache.clock_keys[pos]
            self.assertTrue(clock_key is key)
            clock_ref = cache.clock_refs[pos]

        # All clock_refs must be True or False, nothing else.
        for clock_ref in cache.clock_refs:
            self.assertTrue(clock_ref is True or clock_ref is False)

    def test_size_lessthan_1(self):
        self.assertRaises(ValueError, self._makeOne, 0)

    def test_get(self):
        cache = self._makeOne(1)
        # Must support different types of keys
        self.assertIsNone(cache.get("foo"))
        self.assertIsNone(cache.get(42))
        self.assertIsNone(cache.get(("foo", 42)))
        self.assertIsNone(cache.get(None))
        self.assertIsNone(cache.get(""))
        self.assertIsNone(cache.get(object()))
        # Check if default value is used
        self.assertEqual(cache.get("foo", "bar"), "bar")
        self.assertEqual(cache.get("foo", default="bar"), "bar")

        self.check_cache_is_consistent(cache)

    def test_put(self):
        cache = self._makeOne(8)
        self.check_cache_is_consistent(cache)
        # Must support different types of keys
        cache.put("foo", "FOO")
        cache.put(42, "fortytwo")
        cache.put( ("foo", 42), "tuple_as_key")
        cache.put(None, "None_as_key")
        cache.put("", "empty_string_as_key")
        cache.put(3.141, "float_as_key")
        my_object = object()
        cache.put(my_object, "object_as_key")

        self.check_cache_is_consistent(cache)

        self.assertEqual(cache.get("foo"), "FOO")
        self.assertEqual(cache.get(42), "fortytwo")
        self.assertEqual(cache.get(("foo", 42), "fortytwo"), "tuple_as_key")
        self.assertEqual(cache.get(None), "None_as_key")
        self.assertEqual(cache.get(""), "empty_string_as_key")
        self.assertEqual(cache.get(3.141), "float_as_key")
        self.assertEqual(cache.get(my_object), "object_as_key")

        # put()ing again must overwrite
        cache.put(42, "fortytwo again")
        self.assertEqual(cache.get(42), "fortytwo again")

        self.check_cache_is_consistent(cache)

    def test_invalidate(self):
        cache = self._makeOne(3)
        cache.put("foo", "bar")
        cache.put("FOO", "BAR")

        cache.invalidate("foo")
        self.assertIsNone(cache.get("foo"))
        self.assertEqual(cache.get("FOO"), "BAR")
        self.check_cache_is_consistent(cache)

        cache.invalidate("FOO")
        self.assertIsNone(cache.get("foo"))
        self.assertIsNone(cache.get("FOO"))
        self.assertEqual(cache.data, {})
        self.check_cache_is_consistent(cache)

        cache.put("foo", "bar")
        cache.invalidate("nonexistingkey")
        self.assertEqual(cache.get("foo"), "bar")
        self.assertIsNone(cache.get("FOO"))
        self.check_cache_is_consistent(cache)

    def test_small_cache(self):
        #Cache of size 1 must work
        cache = self._makeOne(1)

        cache.put("foo", "bar")
        self.assertEqual(cache.get("foo"), "bar")
        self.check_cache_is_consistent(cache)

        cache.put("FOO", "BAR")
        self.assertEqual(cache.get("FOO"), "BAR")
        self.assertIsNone(cache.get("foo"))
        self.check_cache_is_consistent(cache)

        # put() again
        cache.put("FOO", "BAR")
        self.assertEqual(cache.get("FOO"), "BAR")
        self.assertIsNone(cache.get("foo"))
        self.check_cache_is_consistent(cache)

        # invalidate()
        cache.invalidate("FOO")
        self.check_cache_is_consistent(cache)
        self.assertIsNone(cache.get("FOO"))
        self.assertIsNone(cache.get("foo"))

        # clear()
        cache.put("foo", "bar")
        self.assertEqual(cache.get("foo"), "bar")
        cache.clear()
        self.check_cache_is_consistent(cache)
        self.assertIsNone(cache.get("FOO"))
        self.assertIsNone(cache.get("foo"))

    def test_equal_but_not_identical(self):
        #equal but not identical keys must be treated the same
        cache = self._makeOne(1)
        tuple_one = (1, 1)
        tuple_two = (1, 1)
        cache.put(tuple_one, 42)

        self.assertEqual(cache.get(tuple_one), 42)
        self.assertEqual(cache.get(tuple_two), 42)
        self.check_cache_is_consistent(cache)

        cache = self._makeOne(1)
        cache.put(tuple_one, 42)
        cache.invalidate(tuple_two)
        self.assertIsNone(cache.get(tuple_one))
        self.assertIsNone(cache.get(tuple_two))

    def test_perfect_hitrate(self):
        #If cache size equals number of items, expect 100% cache hits
        size = 1000
        cache = self._makeOne(size)

        for count in range(size):
            cache.put(count, "item%s" % count)

        for cache_op in range(10000):
            item = random.randrange(0, size - 1)
            if random.getrandbits(1):
                self.assertEqual(cache.get(item), "item%s" % item)
            else:
                cache.put(item, "item%s" % item)

        self.assertEqual(cache.misses, 0)
        self.assertEqual(cache.evictions, 0)

        self.check_cache_is_consistent(cache)

    def test_imperfect_hitrate(self):
        #If cache size == half the number of items -> hit rate ~50%
        size = 1000
        cache = self._makeOne(size / 2)

        for count in range(size):
            cache.put(count, "item%s" % count)

        hits = 0
        misses = 0
        total_gets = 0
        for cache_op in range(10000):
            item = random.randrange(0, size - 1)
            if random.getrandbits(1):
                entry = cache.get(item)
                total_gets += 1
                self.assertTrue(
                        (entry == "item%s" % item) or
                        entry is None)
                if entry is None:
                    misses += 1
                else:
                    hits += 1
            else:
                cache.put(item, "item%s" % item)

        # Cache hit rate should be roughly 50%
        hit_ratio = hits / float(total_gets) * 100
        self.assertTrue(hit_ratio > 45)
        self.assertTrue(hit_ratio < 55)

        # The internal cache counters should have the same information
        internal_hit_ratio = 100 * cache.hits / cache.lookups
        self.assertTrue(internal_hit_ratio > 45)
        self.assertTrue(internal_hit_ratio < 55)

        # The internal miss counters should also be around 50%
        internal_miss_ratio = 100 * cache.misses / cache.lookups
        self.assertTrue(internal_miss_ratio > 45)
        self.assertTrue(internal_miss_ratio < 55)

        self.check_cache_is_consistent(cache)

    def test_eviction_counter(self):
        cache = self._makeOne(2)
        cache.put(1, 1)
        cache.put(2, 1)
        self.assertEqual(cache.evictions, 0)

        cache.put(3, 1)
        cache.put(4, 1)
        self.assertEqual(cache.evictions, 2)

        cache.put(3, 1)
        cache.put(4, 1)
        self.assertEqual(cache.evictions, 2)

        cache.clear()
        self.assertEqual(cache.evictions, 0)


    def test_it(self):
        cache = self._makeOne(3)
        self.assertIsNone(cache.get('a'))

        cache.put('a', '1')
        pos, value = cache.data.get('a')
        self.assertEqual(cache.clock_refs[pos], True)
        self.assertEqual(cache.clock_keys[pos], 'a')
        self.assertEqual(value, '1')
        self.assertEqual(cache.get('a'), '1')
        self.assertEqual(cache.hand, pos + 1)

        pos, value = cache.data.get('a')
        self.assertEqual(cache.clock_refs[pos], True)
        self.assertEqual(cache.hand, pos + 1)
        self.assertEqual(len(cache.data), 1)

        cache.put('b', '2')
        pos, value = cache.data.get('b')
        self.assertEqual(cache.clock_refs[pos], True)
        self.assertEqual(cache.clock_keys[pos], 'b')
        self.assertEqual(len(cache.data), 2)

        cache.put('c', '3')
        pos, value = cache.data.get('c')
        self.assertEqual(cache.clock_refs[pos], True)
        self.assertEqual(cache.clock_keys[pos], 'c')
        self.assertEqual(len(cache.data), 3)

        pos, value = cache.data.get('a')
        self.assertEqual(cache.clock_refs[pos], True)

        cache.get('a')
        # All items have ref==True. cache.hand points to "a". Putting
        # "d" will set ref=False on all items and then replace "a",
        # because "a" is the first item with ref==False that is found.
        cache.put('d', '4')
        self.assertEqual(len(cache.data), 3)
        self.assertIsNone(cache.data.get('a'))

        # Only item "d" has ref==True. cache.hand points at "b", so "b"
        # will be evicted when "e" is inserted. "c" will be left alone.
        cache.put('e', '5')
        self.assertEqual(len(cache.data), 3)
        self.assertIsNone(cache.data.get('b'))
        self.assertEqual(cache.get('d'), '4')
        self.assertEqual(cache.get('e'), '5')
        self.assertIsNone(cache.get('a'))
        self.assertIsNone(cache.get('b'))
        self.assertEqual(cache.get('c'), '3')

        self.check_cache_is_consistent(cache)


class ExpiringLRUCacheTests(LRUCacheTests):

    def _getTargetClass(self):
        from repoze.lru import ExpiringLRUCache
        return ExpiringLRUCache

    def _makeOne(self, size, default_timeout=None):
        if default_timeout is None:
            return self._getTargetClass()(size)
        else:
            return self._getTargetClass()(
                size, default_timeout=default_timeout)

    def check_cache_is_consistent(self, cache):
        #Return if cache is consistent, else raise fail test case.
        #
        #This is slightly different for ExpiringLRUCache since self.data
        #contains 3-tuples instead of 2-tuples.
        # cache.hand/maxpos/size
        self.assertTrue(cache.hand < len(cache.clock_keys))
        self.assertTrue(cache.hand >= 0)
        self.assertEqual(cache.maxpos, cache.size - 1)
        self.assertEqual(len(cache.clock_keys), cache.size)

        # lengths of data structures
        self.assertEqual(len(cache.clock_keys), len(cache.clock_refs))
        self.assertTrue(len(cache.data) <= len(cache.clock_refs))

        # For each item in cache.data
        #   1. pos must be a valid index
        #   2. clock_keys must point back to the entry
        for key, value in cache.data.items():
            pos, val, timeout = value
            self.assertTrue(
                type(pos) == type(42) or type(pos) == type(2 ** 128))
            self.assertTrue(pos >= 0)
            self.assertTrue(pos <= cache.maxpos)

            clock_key = cache.clock_keys[pos]
            self.assertTrue(clock_key is key)
            clock_ref = cache.clock_refs[pos]

            self.assertTrue(type(timeout) == type(3.141))

        # All clock_refs must be True or False, nothing else.
        for clock_ref in cache.clock_refs:
            self.assertTrue(clock_ref is True or clock_ref is False)

    def test_it(self):
        #Test a sequence of operations
        #
        # Looks at internal data, which is different for ExpiringLRUCache.
        cache = self._makeOne(3)
        self.assertIsNone(cache.get('a'))

        cache.put('a', '1')
        pos, value, expires = cache.data.get('a')
        self.assertEqual(cache.clock_refs[pos], True)
        self.assertEqual(cache.clock_keys[pos], 'a')
        self.assertEqual(value, '1')
        self.assertEqual(cache.get('a'), '1')
        self.assertEqual(cache.hand, pos + 1)

        pos, value, expires = cache.data.get('a')
        self.assertEqual(cache.clock_refs[pos], True)
        self.assertEqual(cache.hand, pos + 1)
        self.assertEqual(len(cache.data), 1)

        cache.put('b', '2')
        pos, value, expires = cache.data.get('b')
        self.assertEqual(cache.clock_refs[pos], True)
        self.assertEqual(cache.clock_keys[pos], 'b')
        self.assertEqual(len(cache.data), 2)

        cache.put('c', '3')
        pos, value, expires = cache.data.get('c')
        self.assertEqual(cache.clock_refs[pos], True)
        self.assertEqual(cache.clock_keys[pos], 'c')
        self.assertEqual(len(cache.data), 3)

        pos, value, expires = cache.data.get('a')
        self.assertEqual(cache.clock_refs[pos], True)

        cache.get('a')
        # All items have ref==True. cache.hand points to "a". Putting
        # "d" will set ref=False on all items and then replace "a",
        # because "a" is the first item with ref==False that is found.
        cache.put('d', '4')
        self.assertEqual(len(cache.data), 3)
        self.assertIsNone(cache.data.get('a'))

        # Only item "d" has ref==True. cache.hand points at "b", so "b"
        # will be evicted when "e" is inserted. "c" will be left alone.
        cache.put('e', '5')
        self.assertEqual(len(cache.data), 3)
        self.assertIsNone(cache.data.get('b'))
        self.assertEqual(cache.get('d'), '4')
        self.assertEqual(cache.get('e'), '5')
        self.assertIsNone(cache.get('a'))
        self.assertIsNone(cache.get('b'))
        self.assertEqual(cache.get('c'), '3')

        self.check_cache_is_consistent(cache)

    def test_default_timeout(self):
        #Default timeout provided at init time must be applied.
        # Provide no default timeout -> entries must remain valid
        cache = self._makeOne(3)
        cache.put("foo", "bar")

        time.sleep(0.1)
        cache.put("FOO", "BAR")
        self.assertEqual(cache.get("foo"), "bar")
        self.assertEqual(cache.get("FOO"), "BAR")
        self.check_cache_is_consistent(cache)

        # Provide short default timeout -> entries must become invalid
        cache = self._makeOne(3, default_timeout=0.1)
        cache.put("foo", "bar")

        time.sleep(0.1)
        cache.put("FOO", "BAR")
        self.assertIsNone(cache.get("foo"))
        self.assertEqual(cache.get("FOO"), "BAR")
        self.check_cache_is_consistent(cache)

    def test_different_timeouts(self):
        #Timeouts must be per entry, default applied when none provided
        cache = self._makeOne(3, default_timeout=0.1)

        cache.put("one", 1)
        cache.put("two", 2, timeout=0.2)
        cache.put("three", 3, timeout=0.3)

        # All entries still here
        self.assertEqual(cache.get("one"), 1)
        self.assertEqual(cache.get("two"), 2)
        self.assertEqual(cache.get("three"), 3)

        # Entry "one" must expire, "two"/"three" remain valid
        time.sleep(0.1)
        self.assertIsNone(cache.get("one"))
        self.assertEqual(cache.get("two"), 2)
        self.assertEqual(cache.get("three"), 3)

        # Only "three" remains valid
        time.sleep(0.1)
        self.assertIsNone(cache.get("one"))
        self.assertIsNone(cache.get("two"))
        self.assertEqual(cache.get("three"), 3)

        # All have expired
        time.sleep(0.1)
        self.assertIsNone(cache.get("one"))
        self.assertIsNone(cache.get("two"))
        self.assertIsNone(cache.get("three"))

        self.check_cache_is_consistent(cache)

    def test_renew_timeout(self):
        #Re-putting an entry must update timeout
        cache = self._makeOne(3, default_timeout=0.2)

        cache.put("foo", "bar")
        cache.put("foo2", "bar2", timeout=10)
        cache.put("foo3", "bar3", timeout=10)

        time.sleep(0.1)
        # All must still be here
        self.assertEqual(cache.get("foo"), "bar")
        self.assertEqual(cache.get("foo2"), "bar2")
        self.assertEqual(cache.get("foo3"), "bar3")
        self.check_cache_is_consistent(cache)

        # Set new timeouts by re-put()ing the entries
        cache.put("foo", "bar")
        cache.put("foo2", "bar2", timeout=0.1)
        cache.put("foo3", "bar3")

        time.sleep(0.1)
        # "foo2" must have expired
        self.assertEqual(cache.get("foo"), "bar")
        self.assertIsNone(cache.get("foo2"))
        self.assertEqual(cache.get("foo3"), "bar3")
        self.check_cache_is_consistent(cache)


class DecoratorTests(unittest.TestCase):

    def _getTargetClass(self):
        from repoze.lru import lru_cache
        return lru_cache

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_ctor_no_size(self):
        from repoze.lru import UnboundedCache
        decorator = self._makeOne(maxsize=None)
        self.assertIsInstance(decorator.cache, UnboundedCache)
        self.assertEqual(decorator.cache._data, {})

    def test_ctor_w_size_no_timeout(self):
        from repoze.lru import LRUCache
        decorator = self._makeOne(maxsize=10)
        self.assertIsInstance(decorator.cache, LRUCache)
        self.assertEqual(decorator.cache.size, 10)

    def test_ctor_w_size_w_timeout(self):
        from repoze.lru import ExpiringLRUCache
        decorator = self._makeOne(maxsize=10, timeout=30)
        self.assertIsInstance(decorator.cache, ExpiringLRUCache)
        self.assertEqual(decorator.cache.size, 10)
        self.assertEqual(decorator.cache.default_timeout, 30)

    def test_ctor_nocache(self):
        decorator = self._makeOne(10, None)
        self.assertEqual(decorator.cache.size, 10)

    def test_singlearg(self):
        cache = DummyLRUCache()
        decorator = self._makeOne(0, cache)
        def wrapped(key):
            return key
        decorated = decorator(wrapped)
        result = decorated(1)
        self.assertEqual(cache[(1,)], 1)
        self.assertEqual(result, 1)
        self.assertEqual(len(cache), 1)
        result = decorated(2)
        self.assertEqual(cache[(2,)], 2)
        self.assertEqual(result, 2)
        self.assertEqual(len(cache), 2)
        result = decorated(2)
        self.assertEqual(cache[(2,)], 2)
        self.assertEqual(result, 2)
        self.assertEqual(len(cache), 2)

    def test_cache_attr(self):
        cache = DummyLRUCache()
        decorator = self._makeOne(0, cache)
        def wrapped(key): #pragma NO COVER
            return key
        decorated = decorator(wrapped)
        self.assertTrue(decorated._cache is cache)

    def test_multiargs(self):
        cache = DummyLRUCache()
        decorator = self._makeOne(0, cache)
        def moreargs(*args):
            return args
        decorated = decorator(moreargs)
        result = decorated(3, 4, 5)
        self.assertEqual(cache[(3, 4, 5)], (3, 4, 5))
        self.assertEqual(result, (3, 4, 5))
        self.assertEqual(len(cache), 1)

    def test_multiargs_keywords(self):
        cache = DummyLRUCache()
        decorator = self._makeOne(0, cache)
        def moreargs(*args, **kwargs):
            return args, kwargs
        decorated = decorator(moreargs)
        result = decorated(3, 4, 5, a=1, b=2, c=3)
        self.assertEqual(
            cache[((3, 4, 5), frozenset([ ('a',1), ('b',2), ('c',3) ]))],
            ((3, 4, 5), {'a':1, 'b':2, 'c':3}))
        self.assertEqual(result, ((3, 4, 5), {'a':1, 'b':2, 'c':3}))
        self.assertEqual(len(cache), 1)

    def test_multiargs_keywords_ignore_unhashable_true(self):
        cache = DummyLRUCache()
        decorator = self._makeOne(0, cache, ignore_unhashable_args=True)
        def moreargs(*args, **kwargs):
            return args, kwargs
        decorated = decorator(moreargs)
        result = decorated(3, 4, 5, a=1, b=[1, 2, 3])
        self.assertEqual(len(cache), 0)
        self.assertEqual(result, ((3, 4, 5), {'a':1, 'b':[1, 2, 3]}))

    def test_multiargs_keywords_ignore_unhashable(self):
        cache = DummyLRUCache()
        decorator = self._makeOne(0, cache, ignore_unhashable_args=False)

        def moreargs(*args, **kwargs):  # pragma: NO COVER
            return args, kwargs

        decorated = decorator(moreargs)

        with self.assertRaises(TypeError):
            decorated(3, 4, 5, a=1, b=[1, 2, 3])

    def test_expiry(self):
        #When timeout is given, decorator must eventually forget entries
        @self._makeOne(1, None, timeout=0.1)
        def sleep_a_bit(param):
            time.sleep(0.1)
            return 2 * param

        # First call must take at least 0.1 seconds
        start = time.time()
        result1 = sleep_a_bit("hello")
        stop = time.time()
        self.assertEqual(result1, 2 * "hello")
        self.assertTrue(stop - start > 0.1)

        # Second call must take less than 0.1 seconds.
        start = time.time()
        result2 = sleep_a_bit("hello")
        stop = time.time()
        self.assertEqual(result2, 2 * "hello")
        self.assertTrue(stop - start < 0.1)

        time.sleep(0.1)
        # This one must calculate again and take at least 0.1 seconds
        start = time.time()
        result3 = sleep_a_bit("hello")
        stop = time.time()
        self.assertEqual(result3, 2 * "hello")
        self.assertTrue(stop - start > 0.1)

    def test_partial(self):
        #lru_cache decorator must not crash on functools.partial instances
        def add(a,b):
            return a + b
        from functools import partial
        from repoze.lru import lru_cache
        add_five = partial(add, 5)
        decorated = lru_cache(20)(add_five)
        self.assertEqual(decorated(3), 8)


class DummyLRUCache(dict):

    def put(self, k, v):
        return self.__setitem__(k, v)


class CacherMaker(unittest.TestCase):

    def _getTargetClass(self):
        from repoze.lru import CacheMaker
        return CacheMaker

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_named_cache(self):
        maker = self._makeOne()
        size = 10
        name = "name"
        decorated = maker.lrucache(maxsize=size, name=name)(_adder)
        self.assertEqual(list(maker._cache.keys()), [name])
        self.assertEqual(maker._cache[name].size, size)
        decorated(10)
        decorated(11)
        self.assertEqual(len(maker._cache[name].data),2)

    def test_exception(self):
        maker = self._makeOne()
        size = 10
        name = "name"
        decorated = maker.lrucache(maxsize=size, name=name)(_adder)
        self.assertRaises(KeyError, maker.lrucache, maxsize=size, name=name)
        self.assertRaises(ValueError, maker.lrucache)

    def test_defaultvalue_and_clear(self):
        size = 10
        maker = self._makeOne(maxsize=size)
        for i in range(100):
            decorated = maker.lrucache()(_adder)
            decorated(10)

        self.assertEqual(len(maker._cache) , 100)
        for _cache in maker._cache.values():
            self.assertEqual( _cache.size,size)
            self.assertEqual(len(_cache.data),1)
        ## and test clear cache
        maker.clear()
        for _cache in maker._cache.values():
            self.assertEqual( _cache.size,size)
            self.assertEqual(len(_cache.data),0)

    def test_clear_with_single_name(self):
        maker = self._makeOne(maxsize=10)
        one = maker.lrucache(name='one')(_adder)
        two = maker.lrucache(name='two')(_adder)
        for i in range(100):
            _ = one(i)
            _ = two(i)
        self.assertEqual(len(maker._cache['one'].data), 10)
        self.assertEqual(len(maker._cache['two'].data), 10)
        maker.clear('one')
        self.assertEqual(len(maker._cache['one'].data), 0)
        self.assertEqual(len(maker._cache['two'].data), 10)

    def test_clear_with_multiple_names(self):
        maker = self._makeOne(maxsize=10)
        one = maker.lrucache(name='one')(_adder)
        two = maker.lrucache(name='two')(_adder)
        three = maker.lrucache(name='three')(_adder)
        for i in range(100):
            _ = one(i)
            _ = two(i)
            _ = three(i)
        self.assertEqual(len(maker._cache['one'].data), 10)
        self.assertEqual(len(maker._cache['two'].data), 10)
        self.assertEqual(len(maker._cache['three'].data), 10)
        maker.clear('one', 'three')
        self.assertEqual(len(maker._cache['one'].data), 0)
        self.assertEqual(len(maker._cache['two'].data), 10)
        self.assertEqual(len(maker._cache['three'].data), 0)

    def test_memoized(self):
        from repoze.lru import lru_cache
        from repoze.lru import UnboundedCache
        maker = self._makeOne(maxsize=10)
        memo = maker.memoized('test')
        self.assertIsInstance(memo, lru_cache)
        self.assertIsInstance(memo.cache, UnboundedCache)
        self.assertIs(memo.cache, maker._cache['test'])

    def test_expiring(self):
        size = 10
        timeout = 10
        name = "name"
        cache = self._makeOne(maxsize=size, timeout=timeout)
        for i in range(100):
            if not i:
                decorator = cache.expiring_lrucache(name=name)
                decorated = decorator(_adder)
                self.assertEqual( cache._cache[name].size,size)
            else:
                decorator = cache.expiring_lrucache()
                decorated = decorator(_adder)
                self.assertEqual(decorator.cache.default_timeout, timeout)
            decorated(10)

        self.assertEqual( len(cache._cache) , 100)
        for _cache in cache._cache.values():
            self.assertEqual( _cache.size,size)
            self.assertEqual( _cache.default_timeout,timeout)
            self.assertEqual(len(_cache.data),1)
        ## and test clear cache
        cache.clear()
        for _cache in cache._cache.values():
            self.assertEqual( _cache.size,size)
            self.assertEqual(len(_cache.data),0)

    def test_expiring_w_timeout(self):
        size = 10
        maker_timeout = 10
        timeout = 20
        name = "name"
        cache = self._makeOne(maxsize=size, timeout=maker_timeout)
        decorator = cache.expiring_lrucache(name=name, timeout=20)
        self.assertEqual(decorator.cache.default_timeout, timeout)

def _adder(x):
    return x + 10



# pylint: disable=no-self-use,invalid-name



from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import
import json
import tarfile

import pytest

from allennlp.common.testing import AllenNlpTestCase
from allennlp.data import Token
from allennlp.data.token_indexers import OpenaiTransformerBytePairIndexer
from io import open


class TestOpenaiTransformerBytePairIndexer(AllenNlpTestCase):
    def setUp(self):
        super(TestOpenaiTransformerBytePairIndexer, self).setUp()

        encoder_path = self.TEST_DIR / u'encoder.json'
        bpe_path = self.TEST_DIR / u'vocab.bpe'
        transformer_model_path = self.TEST_DIR / u'model.tar.gz'

        symbols = [u"e", u"w", u"o", u"wo", u"."]
        byte_pairs = [(sym1, sym2 + end)
                      for sym1 in symbols        # prefer earlier first symbol
                      for sym2 in symbols        # if tie, prefer earlier second symbol
                      for end in (u'</w>', u'')]   # if tie, prefer ending a word
        encoding = dict(("{sym1}{sym2}", idx + 1) for idx, (sym1, sym2) in enumerate(byte_pairs))


        with open(encoder_path, u'w') as encoder_file:
            json.dump(encoding, encoder_file)

        with open(bpe_path, u'w') as bpe_file:
            bpe_file.write(u"#version 0.0\n")
            for sym1, sym2 in byte_pairs:
                bpe_file.write("{sym1} {sym2}\n")
            bpe_file.write(u"\n")

        with tarfile.open(transformer_model_path, u'w') as tf:
            tf.add(encoder_path, u'model/encoder_bpe_40000.json')
            tf.add(bpe_path, u'model/vocab_40000.bpe')

        self.indexer = OpenaiTransformerBytePairIndexer(encoding, byte_pairs)

    def test_bpe(self):

        # [e, w, o, e</w>] -> best pair (e, w)
        # [ew, o, e</w>] -> best pair (o, e</w>)
        # [ew, oe</w>] -> done
        token = Token(u"ewoe")
        assert self.indexer.byte_pair_encode(token) == [u'ew', u'oe</w>']

        # Prefer "ew" to "we"
        token = Token(u"ewe")
        assert self.indexer.byte_pair_encode(token) == [u'ew', u'e</w>']

        # Prefer ending a word
        token = Token(u"eee")
        assert self.indexer.byte_pair_encode(token) == [u'e', u'ee</w>']

        # Encodes up to a single symbol when appropriate
        token = Token(u"woe")
        assert self.indexer.byte_pair_encode(token) == [u'woe</w>']

    def test_tokens_to_indices(self):
        tokens = [Token(u'ewoe'), Token(u'woe'), Token(u'ewe'), Token(u'ee')]

        indices = self.indexer.tokens_to_indices(tokens, None, u'test')

        assert set(indices.keys()) == set([u"test", u"test-offsets", u"mask"])

        text_tokens = indices[u'test']
        offsets = indices[u'test-offsets']

        assert text_tokens[:6] == [
                self.indexer.encoder.get(symbol, 0)
                for symbol in [u'ew', u'oe</w>'] + [u'woe</w>'] + [u'ew', u'e</w>'] + [u'ee</w>']
        ]

        assert offsets == [
                1,  # end of first word
                2,  # end of second word
                4,  # end of third word
                5,  # end of last word
        ]

    def test_raises_with_too_long_sentence(self):
        tokens = [Token(u'a') for _ in range(513)]

        with pytest.raises(RuntimeError):
            self.indexer.tokens_to_indices(tokens, None, u'should-fail')




from __future__ import absolute_import
#typing

import torch

from allennlp.nn.decoding.decoder_state import DecoderState

# typevar


class DecoderStep(torch.nn.Module, Generic[StateType]):
    u"""
    A ``DecoderStep`` is a module that assigns scores to state transitions in a transition-based
    decoder.

    The ``DecoderStep`` takes a ``DecoderState`` and outputs a ranked list of next states, ordered
    by the state's score.

    The intention with this class is that a model will implement a subclass of ``DecoderStep`` that
    defines how exactly you want to handle the input and what computations get done at each step of
    decoding, and how states are scored.  This subclass then gets passed to a ``DecoderTrainer`` to
    have its parameters trained.
    """
    def take_step(self,
                  state           ,
                  max_actions      = None,
                  allowed_actions            = None)                   :
        u"""
        The main method in the ``DecoderStep`` API.  This function defines the computation done at
        each step of decoding and returns a ranked list of next states.

        The input state is `grouped`, to allow for efficient computation, but the output states
        should all have a ``group_size`` of 1, to make things easier on the decoding algorithm.
        They will get regrouped later as needed.

        Because of the way we handle grouping in the decoder states, constructing a new state is
        actually a relatively expensive operation.  If you know a priori that only some of the
        states will be needed (either because you have a set of gold action sequences, or you have
        a fixed beam size), passing that information into this function will keep us from
        constructing more states than we need, which will greatly speed up your computation.

        IMPORTANT: This method `must` returns states already sorted by their score, otherwise
        ``BeamSearch`` and other methods will break.  For efficiency, we do not perform an
        additional sort in those methods.

        Parameters
        ----------
        state : ``DecoderState``
            The current state of the decoder, which we will take a step `from`.  We may be grouping
            together computation for several states here.  Because we can have several states for
            each instance in the original batch being evaluated at the same time, we use
            ``group_size`` for this kind of batching, and ``batch_size`` for the `original` batch
            in ``model.forward.``
        max_actions : ``int``, optional
            If you know that you will only need a certain number of states out of this (e.g., in a
            beam search), you can pass in the max number of actions that you need, and we will only
            construct that many states (for each `batch` instance - `not` for each `group`
            instance!).  This can save a whole lot of computation if you have an action space
            that's much larger than your beam size.
        allowed_actions : ``List[Set]``, optional
            If the ``DecoderTrainer`` has constraints on which actions need to be evaluated (e.g.,
            maximum marginal likelihood only needs to evaluate action sequences in a given set),
            you can pass those constraints here, to avoid constructing state objects unnecessarily.
            If there are no constraints from the trainer, passing a value of ``None`` here will
            allow all actions to be considered.

            This is a list because it is `batched` - every instance in the batch has a set of
            allowed actions.  Note that the size of this list is the ``group_size`` in the
            ``DecoderState``, `not` the ``batch_size`` of ``model.forward``.  The training
            algorithm needs to convert from the `batched` allowed action sequences that it has to a
            `grouped` allowed action sequence list.

        Returns
        -------
        next_states : ``List[DecoderState]``
            A list of next states, ordered by score.
        """
        raise NotImplementedError



# pylint: disable=no-self-use,invalid-name,line-too-long,no-member




from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
import json
import os
import pathlib
import sys

from allennlp.common.testing import AllenNlpTestCase
from allennlp.service import config_explorer
from allennlp.service.config_explorer import make_app, _HTML
from io import open


class TestConfigExplorer(AllenNlpTestCase):

    def setUp(self):
        super(TestConfigExplorer, self).setUp()
        app = make_app()
        app.testing = True
        self.client = app.test_client()

    def test_html(self):
        u"""
        The pip-installed version of allennlp (currently) requires the config explorer HTML
        to be hardcoded into the server file. But when iterating on it, it's easier to use the
        /debug/ endpoint, which points at `config_explorer.html`, so that you don't have to
        restart the server every time you make a change.

        This test just ensures that the two HTML versions are identical, to prevent you from
        making a change to the standalone HTML but forgetting to change the corresponding
        server HTML. There is certainly a better way to handle this.
        """
        config_explorer_dir = pathlib.Path(config_explorer.__file__).parent
        config_explorer_file = config_explorer_dir / u'config_explorer.html'

        if not config_explorer_file.exists():
            print(u"standalone config_explorer.html does not exist, skipping test")
        else:
            with open(config_explorer_file) as f:
                html = f.read()

            assert html.strip() == _HTML.strip()

    def test_app(self):
        response = self.client.get(u'/')
        html = response.get_data().decode(u'utf-8')

        assert u"AllenNLP Configuration Wizard" in html

    def test_api(self):
        response = self.client.get(u'/api/config/')
        data = json.loads(response.get_data())

        assert data[u"className"] == u""

        items = data[u"config"][u'items']

        assert items[0] == {
                u"name": u"dataset_reader",
                u"configurable": True,
                u"comment": u"specify your dataset reader here",
                u"annotation": {u'origin': u"allennlp.data.dataset_readers.dataset_reader.DatasetReader"}
        }


    def test_choices(self):
        response = self.client.get(u'/api/config/?class=allennlp.data.dataset_readers.dataset_reader.DatasetReader')
        data = json.loads(response.get_data())

        assert u"allennlp.data.dataset_readers.reading_comprehension.squad.SquadReader" in data[u"choices"]

    def test_subclass(self):
        response = self.client.get(u'/api/config/?class=allennlp.data.dataset_readers.semantic_role_labeling.SrlReader')
        data = json.loads(response.get_data())

        config = data[u'config']
        items = config[u'items']
        assert config[u'type'] == u'srl'
        assert items[0][u"name"] == u"token_indexers"

    def test_torch_class(self):
        response = self.client.get(u'/api/config/?class=torch.optim.rmsprop.RMSprop')
        data = json.loads(response.get_data())
        config = data[u'config']
        items = config[u'items']

        assert config[u"type"] == u"rmsprop"
        assert any(item[u"name"] == u"lr" for item in items)

    def test_rnn_hack(self):
        u"""
        Behind the scenes, when you try to create a torch RNN,
        it just calls torch.RNNBase with an extra parameter.
        This test is to make sure that works correctly.
        """
        response = self.client.get(u'/api/config/?class=torch.nn.modules.rnn.LSTM')
        data = json.loads(response.get_data())
        config = data[u'config']
        items = config[u'items']

        assert config[u"type"] == u"lstm"
        assert any(item[u"name"] == u"batch_first" for item in items)

    def test_initializers(self):
        response = self.client.get(u'/api/config/?class=allennlp.nn.initializers.Initializer')
        data = json.loads(response.get_data())

        assert u'torch.nn.init.constant_' in data[u"choices"]
        assert u'allennlp.nn.initializers.block_orthogonal' in data[u"choices"]

        response = self.client.get(u'/api/config/?class=torch.nn.init.uniform_')
        data = json.loads(response.get_data())
        config = data[u'config']
        items = config[u'items']

        assert config[u"type"] == u"uniform"
        assert any(item[u"name"] == u"a" for item in items)

    def test_regularizers(self):
        response = self.client.get(u'/api/config/?class=allennlp.nn.regularizers.regularizer.Regularizer')
        data = json.loads(response.get_data())

        assert u'allennlp.nn.regularizers.regularizers.L1Regularizer' in data[u"choices"]

        response = self.client.get(u'/api/config/?class=allennlp.nn.regularizers.regularizers.L1Regularizer')
        data = json.loads(response.get_data())
        config = data[u'config']
        items = config[u'items']

        assert config[u"type"] == u"l1"
        assert any(item[u"name"] == u"alpha" for item in items)

    def test_other_modules(self):
        # Create a new package in a temporary dir
        packagedir = self.TEST_DIR / u'configexplorer'
        packagedir.mkdir()  # pylint: disable=no-member
        (packagedir / u'__init__.py').touch()  # pylint: disable=no-member

        # And add that directory to the path
        sys.path.insert(0, unicode(self.TEST_DIR))

        # Write out a duplicate predictor there, but registered under a different name.
        from allennlp.predictors import bidaf
        with open(bidaf.__file__) as f:
            code = f.read().replace(u"""@Predictor.register('machine-comprehension')""",
                                    u"""@Predictor.register('config-explorer-predictor')""")

        with open(os.path.join(packagedir, u'predictor.py'), u'w') as f:
            f.write(code)

        # Without specifying modules to load, it shouldn't be there
        app = make_app()
        app.testing = True
        client = app.test_client()
        response = client.get(u'/api/config/?class=allennlp.predictors.predictor.Predictor')
        data = json.loads(response.get_data())
        assert u"allennlp.predictors.bidaf.BidafPredictor" in data[u"choices"]
        assert u"configexplorer.predictor.BidafPredictor" not in data[u"choices"]

        # With specifying extra modules, it should be there.
        app = make_app([u'configexplorer'])
        app.testing = True
        client = app.test_client()
        response = client.get(u'/api/config/?class=allennlp.predictors.predictor.Predictor')
        data = json.loads(response.get_data())
        assert u"allennlp.predictors.bidaf.BidafPredictor" in data[u"choices"]
        assert u"configexplorer.predictor.BidafPredictor" in data[u"choices"]

        sys.path.remove(unicode(self.TEST_DIR))



from __future__ import print_function

import hashlib
import os
import shutil
import sys
import subprocess
import traceback
import tempfile
import zipfile

from glob import glob
from setuptools import find_packages
from distutils.core import setup
from setuptools.command.install import install
from setuptools.command.egg_info import egg_info
from setuptools import setup, Distribution
from multiprocessing import Process

try:
    import pip._internal.pep425tags as pep425tags
    pep425tags.get_supported()
    raise Exception()
except Exception as e:
    import pep425tags

try:
    from urllib.request import urlretrieve
except BaseException:
    from urllib import urlretrieve

PACKAGE_NAME = 'pymagnitude'

# Setup path constants
PROJ_PATH = os.path.abspath(os.path.dirname(os.path.realpath(__file__)))
THIRD_PARTY = PROJ_PATH + '/pymagnitude/third_party'
BUILD_THIRD_PARTY = PROJ_PATH + '/build/lib/pymagnitude/third_party'
PYSQLITE = THIRD_PARTY + '/_pysqlite'
APSW_TP = THIRD_PARTY + '/_apsw'
INTERNAL = THIRD_PARTY + '/internal'
PYSQLITE2 = INTERNAL + '/pysqlite2'
APSW = INTERNAL + '/apsw'

# Get the package version
__version__ = None
with open(os.path.join(PROJ_PATH, 'version.py')) as f:
    exec(f.read())

# Setup remote wheel configurations
RM_WHEELHOUSE = 'https://s3.amazonaws.com/magnitude.plasticity.ai/wheelhouse/'
INSTALLED_FROM_WHEEL = os.path.join(
    tempfile.gettempdir(),
    PACKAGE_NAME +
    '-' +
    __version__ +
    '-' +
    hashlib.md5(PROJ_PATH.encode('utf-8')).hexdigest() +
    '.whlinstall'
)


def try_list_dir(d):
    try:
        return os.listdir(d)
    except BaseException:
        return []


def get_supported_wheels(package_name=PACKAGE_NAME, version=__version__):
    """Get supported wheel strings"""
    def tuple_invalid(t):
        return (
            t[1] == 'none' or
            'fat32' in t[2] or
            'fat64' in t[2] or
            '_universal' in t[2]
        )
    return ['-'.join((package_name, version) + t) + '.whl'
            for t in pep425tags.get_supported() if not(tuple_invalid(t))]


def install_wheel(whl):
    """Installs a wheel file"""
    rc = subprocess.Popen([
        sys.executable,
        '-m',
        'pip',
        'install',
        whl,
    ]).wait()
    return rc


def skip_wheel():
    """ Checks if a wheel install should be skipped """
    return "SKIP_MAGNITUDE_WHEEL" in os.environ


def installed_wheel():
    """Checks if a pre-compiled remote wheel was installed"""
    return os.path.exists(INSTALLED_FROM_WHEEL)


def download_and_install_wheel():
    """Downloads and installs pre-compiled remote wheels"""
    if skip_wheel():
        return False
    if installed_wheel():
        return True
    print("Downloading and installing wheel (if it exists)...")
    tmpwhl_dir = tempfile.gettempdir()
    for whl in get_supported_wheels():
        exitcodes = []
        whl_url = RM_WHEELHOUSE + whl
        dl_path = os.path.join(tmpwhl_dir, whl)
        try:
            urlretrieve(whl_url, dl_path)
        except BaseException:
            continue
        extract_dir = os.path.join(
            tempfile.gettempdir(), whl.replace(
                '.whl', ''))
        extract_dir = os.path.join(
            tempfile.gettempdir(), whl.replace(
                '.whl', ''))
        try:
            zip_ref = zipfile.ZipFile(dl_path, 'r')
        except BaseException:
            continue
        zip_ref.extractall(extract_dir)
        for ewhl in glob(extract_dir + "/*/req_wheels/*.whl"):
            print("Installing requirement wheel: ", ewhl)
            exitcodes.append(install_wheel(ewhl))
        print("Installing wheel: ", dl_path)
        exitcodes.append(install_wheel(dl_path))
        zip_ref.extractall(PROJ_PATH)
        zip_ref.close()
        if len(exitcodes) > 0 and max(exitcodes) == 0 and min(exitcodes) == 0:
            print("Done downloading and installing wheel")
            return True
    print("Done trying to download and install wheel (it didn't exist)")
    return False


def parse_requirements(filename):
    """ load requirements from a pip requirements file """
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


def custom_sqlite3_build():
    """ Checks if custom SQLite has been built already """
    so_files = glob(INTERNAL + '/pysqlite2/*.so')
    pyd_files = glob(INTERNAL + '/pysqlite2/*.pyd')
    return len(so_files + pyd_files) > 0


def install_custom_sqlite3():
    """ Begin install custom SQLite
    Can be safely ignored even if it fails, however, system SQLite
    imitations may prevent large .magnitude files with many columns
    from working."""
    print("Installing custom SQLite 3 (pysqlite) ....")
    install_env = os.environ.copy()
    install_env["PYTHONPATH"] = INTERNAL + \
        (':' + install_env["PYTHONPATH"] if "PYTHONPATH" in install_env else "")
    shutil.copy(
        os.path.join(
            PYSQLITE, 'sqlite3.c'), os.path.join(
            APSW_TP, 'src', 'sqlite3.c'))
    shutil.copy(
        os.path.join(
            PYSQLITE, 'sqlite3.h'), os.path.join(
            APSW_TP, 'src', 'sqlite3.h'))
    rc = subprocess.Popen([
        sys.executable,
        PYSQLITE + '/setup.py',
        'install',
        '--install-lib=' + INTERNAL,
    ], cwd=PYSQLITE, env=install_env).wait()
    if rc:
        print("")
        print("============================================================")
        print("=========================WARNING============================")
        print("============================================================")
        print("It seems like building a custom version of SQLite on your")
        print("machine has failed. This is fine, Magnitude will likely work")
        print("just fine with the sytem version of SQLite for most use cases.")
        print("However, if you are trying to load extremely high dimensional")
        print("models > 999 dimensions, you may run in to SQLite limitations")
        print("that can only be resolved by using the custom version of SQLite.")
        print("To troubleshoot make sure you have appropriate build tools on")
        print("your machine for building C programs like GCC and the standard")
        print("library. Also make sure you have the python-dev development")
        print("libraries and headers for building Python C extensions.")
        print("If you need more help with this, please reach out to ")
        print("opensource@plasticity.ai.")
        print("============================================================")
        print("============================================================")
        print("")
    else:
        print("")
        print("============================================================")
        print("=========================SUCCESS============================")
        print("============================================================")
        print("Building a custom version of SQLite on your machine has")
        print("succeeded.")
        print("Listing internal...")
        print(try_list_dir(INTERNAL))
        print("Listing internal/pysqlite2...")
        print(try_list_dir(PYSQLITE2))
        print("============================================================")
        print("============================================================")
        print("")
    print("Installing custom SQLite 3 (apsw) ....")
    rc = subprocess.Popen([
        sys.executable,
        APSW_TP + '/setup.py',
        'install',
        '--install-lib=' + INTERNAL,
    ], cwd=APSW_TP, env=install_env).wait()
    if rc:
        print("")
        print("============================================================")
        print("=========================WARNING============================")
        print("============================================================")
        print("It seems like building a custom version of SQLite on your")
        print("machine has failed. This is fine, Magnitude will likely work")
        print("just fine with the sytem version of SQLite for most use cases.")
        print("However, if you are trying to stream a remote model that")
        print("can only be resolved by using the custom version of SQLite.")
        print("To troubleshoot make sure you have appropriate build tools on")
        print("your machine for building C programs like GCC and the standard")
        print("library. Also make sure you have the python-dev development")
        print("libraries and headers for building Python C extensions.")
        print("If you need more help with this, please reach out to ")
        print("opensource@plasticity.ai.")
        print("============================================================")
        print("============================================================")
        print("")
    else:
        print("")
        print("============================================================")
        print("=========================SUCCESS============================")
        print("============================================================")
        print("Building a custom version of SQLite on your machine has")
        print("succeeded.")
        print("Listing internal...")
        print(try_list_dir(INTERNAL))
        print("Listing internal/apsw...")
        print(try_list_dir(APSW))
        print("============================================================")
        print("============================================================")
        print("")
        if not(os.path.exists(APSW)):
            print("Install-lib did not install APSW, installing from egg...")
            for egg in glob(INTERNAL + "/apsw-*.egg"):
                if (os.path.isfile(egg)):
                    print("Found an egg file, extracting...")
                    try:
                        zip_ref = zipfile.ZipFile(egg, 'r')
                    except BaseException:
                        print("Egg extraction error")
                        continue
                    zip_ref.extractall(APSW)
                else:
                    print("Found an egg folder, renaming...")
                    os.rename(egg, APSW)
                print("Renaming apsw.py to __init__.py")
                os.rename(
                    os.path.join(
                        APSW, 'apsw.py'), os.path.join(
                        APSW, '__init__.py'))


def build_req_wheels():
    """Builds requirement wheels"""
    print("Building requirements wheels...")
    rc = subprocess.Popen([
        sys.executable,
        '-m',
        'pip',
        'wheel',
        '-r',
        'requirements.txt',
        '--wheel-dir=pymagnitude/req_wheels'
    ], cwd=PROJ_PATH).wait()

    download_req_wheels = [
        ('http://download.pytorch.org/whl/cpu/', 'torch', '0.4.1'),
        ('http://download.pytorch.org/whl/cpu/', 'torch', '0.4.1.post2')
    ]

    for wheelhouse, package, version in download_req_wheels:
        for whl in get_supported_wheels(package, version):
            exitcodes = []
            whl_url = wheelhouse + whl
            sys.stdout.write("Trying to download... '" + whl_url + "'")
            dl_path = os.path.join('pymagnitude/req_wheels', whl)
            try:
                urlretrieve(whl_url, dl_path)
                zip_ref = zipfile.ZipFile(dl_path, 'r')
                sys.stdout.write(" ...SUCCESS\n")
            except BaseException:
                if os.path.exists(dl_path):
                    os.remove(dl_path)
                sys.stdout.write(" ...FAIL\n")
                continue
            sys.stdout.flush()

    if rc:
        print("Failed to build requirements wheels!")
        pass


def install_req_wheels():
    """Installs requirement wheels"""
    print("Installing requirements wheels...")
    for whl in glob('pymagnitude/req_wheels/*.whl'):
        rc = subprocess.Popen([
            sys.executable,
            '-m',
            'pip',
            'install',
            '--ignore-installed',
            whl,
        ], cwd=PROJ_PATH).wait()
    print("Done installing requirements wheels")


def install_requirements():
    """Installs requirements.txt"""
    print("Installing requirements...")
    rc = subprocess.Popen([
        sys.executable,
        '-m',
        'pip',
        'install',
        '-r',
        'requirements.txt'
    ], cwd=PROJ_PATH).wait()
    if rc:
        print("Failed to install some requirements!")
    print("Done installing requirements")


def copy_custom_sqlite3():
    """Copy the pysqlite2 folder into site-packages under
    pymagnitude/third_party/internal/ and
    ./build/lib/pymagnitude/third_party/internal/
    for good measure"""
    from distutils.dir_util import copy_tree
    try:
        import site
        cp_from = INTERNAL + '/'
        if hasattr(site, 'getsitepackages'):
            site_packages = site.getsitepackages()
        else:
            from distutils.sysconfig import get_python_lib
            site_packages = [get_python_lib()]
        for sitepack in site_packages:
            for globbed in glob(sitepack + '/pymagnitude*/'):
                try:
                    cp_to = globbed + '/pymagnitude/third_party/internal/'
                except IndexError as e:
                    print(
                        "Site Package: '" +
                        sitepack +
                        "' did not have pymagnitude")
                    continue
                print("Copying from: ", cp_from, " --> to: ", cp_to)
                copy_tree(cp_from, cp_to)
    except Exception as e:
        print("Error copying internal pysqlite folder to site packages:")
        traceback.print_exc(e)
    try:
        cp_from = INTERNAL + '/'
        cp_to = BUILD_THIRD_PARTY + '/internal/'
        print("Copying from: ", cp_from, " --> to: ", cp_to)
        copy_tree(cp_from, cp_to)
    except Exception as e:
        print("Error copying internal pysqlite folder to build folder:")
        traceback.print_exc(e)


cmdclass = {}

try:
    from wheel.bdist_wheel import bdist_wheel as bdist_wheel_

    class CustomBdistWheelCommand(bdist_wheel_):
        def run(self):
            if not(download_and_install_wheel()):
                install_custom_sqlite3()
                build_req_wheels()
            print("Running wheel...")
            bdist_wheel_.run(self)
            print("Done running wheel")
            copy_custom_sqlite3()

    cmdclass['bdist_wheel'] = CustomBdistWheelCommand

except ImportError as e:
    pass


class CustomInstallCommand(install):
    def run(self):
        if not(download_and_install_wheel()):
            install_custom_sqlite3()
            install_req_wheels()
        print("Running install...")
        p = Process(target=install.run, args=(self,))
        p.start()
        p.join()
        print("Done running install")
        if not(download_and_install_wheel()):
            print("Running egg_install...")
            p = Process(target=install.do_egg_install, args=(self,))
            p.start()
            p.join()
            install_requirements()
            print("Done running egg_install")
        else:
            print("Skipping egg_install")
        copy_custom_sqlite3()

    def finalize_options(self):
        install.finalize_options(self)
        if self.distribution.has_ext_modules():
            self.install_lib = self.install_platlib


cmdclass['install'] = CustomInstallCommand


class BinaryDistribution(Distribution):
    def has_ext_modules(foo):
        return True


if __name__ == '__main__':

    # Attempt to install from a remote pre-compiled wheel
    if any([a in sys.argv for a in ['egg_info', 'install']]):
        if download_and_install_wheel():
            open(INSTALLED_FROM_WHEEL, 'w+').close()

    # Only create requirements if not installing from a wheel
    if not(installed_wheel()):
        reqs = parse_requirements('requirements.txt')
    else:
        reqs = []

    setup(
        name=PACKAGE_NAME,
        packages=find_packages(
            exclude=[
                'tests',
                'tests.*']),
        version=__version__,
        description='A fast, efficient universal vector embedding utility package.',
        long_description="""
    About
    -----
    A feature-packed Python package and vector storage file format for utilizing vector embeddings in machine learning models in a fast, efficient, and simple manner developed by `Plasticity <https://www.plasticity.ai/>`_. It is primarily intended to be a faster alternative to `Gensim <https://radimrehurek.com/gensim/>`_, but can be used as a generic key-vector store for domains outside NLP.

    Documentation
    -------------
    You can see the full documentation and README at the `GitLab repository <https://gitlab.com/Plasticity/magnitude>`_ or the `GitHub repository <https://github.com/plasticityai/magnitude>`_.
        """,
        author='Plasticity',
        author_email='opensource@plasticity.ai',
        url='https://gitlab.com/Plasticity/magnitude',
        keywords=[
                    'pymagnitude',
                    'magnitude',
                    'plasticity',
                    'nlp',
                    'natural',
                    'language',
                    'processing',
                    'word',
                    'vector',
                    'embeddings',
                    'embedding',
                    'word2vec',
                    'gensim',
                    'alternative',
                    'machine',
                    'learning',
                    'annoy',
                    'index',
                    'approximate',
                    'nearest',
                    'neighbors'],
        license='MIT',
        include_package_data=True,
        install_requires=reqs,
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            'Intended Audience :: Developers',
            "Topic :: Software Development :: Libraries :: Python Modules",
            'Topic :: Scientific/Engineering :: Artificial Intelligence',
            'Topic :: Text Processing :: Linguistic',
            "Operating System :: OS Independent",
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.0',
            'Programming Language :: Python :: 3.7'],
        cmdclass=cmdclass,
        distclass=BinaryDistribution,
    )



# pylint: disable=no-self-use,invalid-name


from __future__ import division
from __future__ import absolute_import
import pytest

from allennlp.data.dataset_readers import LanguageModelingReader
from allennlp.common.util import ensure_list
from allennlp.common.testing import AllenNlpTestCase

class TestLanguageModelingDatasetReader(object):
    @pytest.mark.parametrize(u"lazy", (True, False))
    def test_read_from_file(self, lazy):
        reader = LanguageModelingReader(tokens_per_instance=3, lazy=lazy)

        instances = ensure_list(reader.read(AllenNlpTestCase.FIXTURES_ROOT / u'data' / u'language_modeling.txt'))
        # The last potential instance is left out, which is ok, because we don't have an end token
        # in here, anyway.
        assert len(instances) == 5

        assert [t.text for t in instances[0].fields[u"input_tokens"].tokens] == [u"This", u"is", u"a"]
        assert [t.text for t in instances[0].fields[u"output_tokens"].tokens] == [u"is", u"a", u"sentence"]

        assert [t.text for t in instances[1].fields[u"input_tokens"].tokens] == [u"sentence", u"for", u"language"]
        assert [t.text for t in instances[1].fields[u"output_tokens"].tokens] == [u"for", u"language", u"modelling"]

        assert [t.text for t in instances[2].fields[u"input_tokens"].tokens] == [u"modelling", u".", u"Here"]
        assert [t.text for t in instances[2].fields[u"output_tokens"].tokens] == [u".", u"Here", u"'s"]

        assert [t.text for t in instances[3].fields[u"input_tokens"].tokens] == [u"'s", u"another", u"one"]
        assert [t.text for t in instances[3].fields[u"output_tokens"].tokens] == [u"another", u"one", u"for"]

        assert [t.text for t in instances[4].fields[u"input_tokens"].tokens] == [u"for", u"extra", u"language"]
        assert [t.text for t in instances[4].fields[u"output_tokens"].tokens] == [u"extra", u"language", u"modelling"]



u"""
A stacked LSTM with LSTM layers which alternate between going forwards over
the sequence and going backwards.
"""


from __future__ import absolute_import
#typing
import torch
from torch.nn.utils.rnn import PackedSequence
from allennlp.modules.augmented_lstm import AugmentedLstm
from allennlp.common.checks import ConfigurationError
try:
    from itertools import izip
except:
    izip = zip



class StackedAlternatingLstm(torch.nn.Module):
    u"""
    A stacked LSTM with LSTM layers which alternate between going forwards over
    the sequence and going backwards. This implementation is based on the
    description in `Deep Semantic Role Labelling - What works and what's next
    <https://homes.cs.washington.edu/~luheng/files/acl2017_hllz.pdf>`_ .

    Parameters
    ----------
    input_size : int, required
        The dimension of the inputs to the LSTM.
    hidden_size : int, required
        The dimension of the outputs of the LSTM.
    num_layers : int, required
        The number of stacked LSTMs to use.
    recurrent_dropout_probability: float, optional (default = 0.0)
        The dropout probability to be used in a dropout scheme as stated in
        `A Theoretically Grounded Application of Dropout in Recurrent Neural Networks
        <https://arxiv.org/abs/1512.05287>`_ .
    use_input_projection_bias : bool, optional (default = True)
        Whether or not to use a bias on the input projection layer. This is mainly here
        for backwards compatibility reasons and will be removed (and set to False)
        in future releases.

    Returns
    -------
    output_accumulator : PackedSequence
        The outputs of the interleaved LSTMs per timestep. A tensor of shape
        (batch_size, max_timesteps, hidden_size) where for a given batch
        element, all outputs past the sequence length for that batch are
        zero tensors.
    """
    def __init__(self,
                 input_size     ,
                 hidden_size     ,
                 num_layers     ,
                 recurrent_dropout_probability        = 0.0,
                 use_highway       = True,
                 use_input_projection_bias       = True)        :
        super(StackedAlternatingLstm, self).__init__()

        # Required to be wrapped with a :class:`PytorchSeq2SeqWrapper`.
        self.input_size = input_size
        self.hidden_size = hidden_size
        self.num_layers = num_layers

        layers = []
        lstm_input_size = input_size
        for layer_index in range(num_layers):
            go_forward = True if layer_index % 2 == 0 else False
            layer = AugmentedLstm(lstm_input_size, hidden_size, go_forward,
                                  recurrent_dropout_probability=recurrent_dropout_probability,
                                  use_highway=use_highway,
                                  use_input_projection_bias=use_input_projection_bias)
            lstm_input_size = hidden_size
            self.add_module(u'layer_{}'.format(layer_index), layer)
            layers.append(layer)
        self.lstm_layers = layers

    def forward(self,  # pylint: disable=arguments-differ
                inputs                ,
                initial_state                                              = None):
        u"""
        Parameters
        ----------
        inputs : ``PackedSequence``, required.
            A batch first ``PackedSequence`` to run the stacked LSTM over.
        initial_state : Tuple[torch.Tensor, torch.Tensor], optional, (default = None)
            A tuple (state, memory) representing the initial hidden state and memory
            of the LSTM. Each tensor has shape (1, batch_size, output_dimension).

        Returns
        -------
        output_sequence : PackedSequence
            The encoded sequence of shape (batch_size, sequence_length, hidden_size)
        final_states: torch.Tensor
            The per-layer final (state, memory) states of the LSTM, each with shape
            (num_layers, batch_size, hidden_size).
        """
        if not initial_state:
            hidden_states = [None] * len(self.lstm_layers)
        elif initial_state[0].size()[0] != len(self.lstm_layers):
            raise ConfigurationError(u"Initial states were passed to forward() but the number of "
                                     u"initial states does not match the number of layers.")
        else:
            hidden_states = list(izip(initial_state[0].split(1, 0),
                                     initial_state[1].split(1, 0)))

        output_sequence = inputs
        final_states = []
        for i, state in enumerate(hidden_states):
            layer = getattr(self, u'layer_{}'.format(i))
            # The state is duplicated to mirror the Pytorch API for LSTMs.
            output_sequence, final_state = layer(output_sequence, state)
            final_states.append(final_state)

        final_state_tuple = (torch.cat(state_list, 0) for state_list in izip(*final_states))
        return output_sequence, final_state_tuple



# pylint: disable=no-self-use,invalid-name,bad-continuation



from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import
import json
import os
import re
import tempfile
from collections import OrderedDict

import pytest

from allennlp.common.params import Params, unflatten, with_fallback, parse_overrides
from allennlp.common.testing import AllenNlpTestCase
from io import open


class TestParams(AllenNlpTestCase):

    def test_load_from_file(self):
        filename = self.FIXTURES_ROOT / u'bidaf' / u'experiment.json'
        params = Params.from_file(filename)

        assert u"dataset_reader" in params
        assert u"trainer" in params

        model_params = params.pop(u"model")
        assert model_params.pop(u"type") == u"bidaf"

    def test_overrides(self):
        filename = self.FIXTURES_ROOT / u'bidaf' / u'experiment.json'
        overrides = u'{ "train_data_path": "FOO", "model": { "type": "BAR" },'\
                    u'"model.text_field_embedder.tokens.type": "BAZ" }'
        params = Params.from_file(filename, overrides)

        assert u"dataset_reader" in params
        assert u"trainer" in params
        assert params[u"train_data_path"] == u"FOO"

        model_params = params.pop(u"model")
        assert model_params.pop(u"type") == u"BAR"
        assert model_params[u"text_field_embedder"][u"tokens"][u"type"] == u"BAZ"

    def test_unflatten(self):
        flattened = {u"a.b.c": 1, u"a.b.d": 0, u"a.e.f.g.h": 2, u"b": 3}
        unflattened = unflatten(flattened)
        assert unflattened == {
            u"a": {
                u"b": {
                    u"c": 1,
                    u"d": 0
                },
                u"e": {
                    u"f": {
                        u"g": {
                            u"h": 2
                        }
                    }
                }
            },
            u"b": 3
        }

        # should do nothing to a non-flat dictionary
        assert unflatten(unflattened) == unflattened

    def test_with_fallback(self):
        preferred = {u"a": 1}
        fallback = {u"a": 0, u"b": 2}

        merged = with_fallback(preferred=preferred, fallback=fallback)
        assert merged == {u"a": 1, u"b": 2}

        # incompatibility is ok
        preferred = {u"a": {u"c": 3}}
        fallback = {u"a": 0, u"b": 2}
        merged = with_fallback(preferred=preferred, fallback=fallback)
        assert merged == {u"a": {u"c": 3}, u"b": 2}

        # goes deep
        preferred = {u"deep": {u"a": 1}}
        fallback = {u"deep": {u"a": 0, u"b": 2}}

        merged = with_fallback(preferred=preferred, fallback=fallback)
        assert merged == {u"deep": {u"a": 1, u"b": 2}}

    def test_parse_overrides(self):
        assert parse_overrides(u"") == {}
        assert parse_overrides(u"{}") == {}

        override_dict = parse_overrides(u'{"train_data": "/train", "trainer.num_epochs": 10}')
        assert override_dict == {
            u"train_data": u"/train",
            u"trainer": {
                u"num_epochs": 10
            }
        }

        params = with_fallback(
            preferred=override_dict,
            fallback={
                u"train_data": u"/test",
                u"model": u"bidaf",
                u"trainer": {u"num_epochs": 100, u"optimizer": u"sgd"}
            })

        assert params == {
            u"train_data": u"/train",
            u"model": u"bidaf",
            u"trainer": {u"num_epochs": 10, u"optimizer": u"sgd"}
        }

    def test_as_flat_dict(self):
        params = Params({
                u'a': 10,
                u'b': {
                        u'c': 20,
                        u'd': u'stuff'
                }
        }).as_flat_dict()

        assert params == {u'a': 10, u'b.c': 20, u'b.d': u'stuff'}

    def test_jsonnet_features(self):
        config_file = self.TEST_DIR / u'config.jsonnet'
        with open(config_file, u'w') as f:
            f.write(u"""{
                            // This example is copied straight from the jsonnet docs
                            person1: {
                                name: "Alice",
                                welcome: "Hello " + self.name + "!",
                            },
                            person2: self.person1 { name: "Bob" },
                        }""")

        params = Params.from_file(config_file)

        alice = params.pop(u"person1")
        bob = params.pop(u"person2")

        assert alice.as_dict() == {u"name": u"Alice", u"welcome": u"Hello Alice!"}
        assert bob.as_dict() == {u"name": u"Bob", u"welcome": u"Hello Bob!"}

        params.assert_empty(u"TestParams")


    def test_regexes_with_backslashes(self):
        bad_regex = self.TEST_DIR / u'bad_regex.jsonnet'
        good_regex = self.TEST_DIR / u'good_regex.jsonnet'

        with open(bad_regex, u'w') as f:
            f.write(ur'{"myRegex": "a\.b"}')

        with open(good_regex, u'w') as f:
            f.write(ur'{"myRegex": "a\\.b"}')

        with pytest.raises(RuntimeError):
            Params.from_file(bad_regex)

        params = Params.from_file(good_regex)
        regex = params[u'myRegex']

        assert re.match(regex, u"a.b")
        assert not re.match(regex, u"a-b")

        # Check roundtripping
        good_regex2 = self.TEST_DIR / u'good_regex2.jsonnet'
        with open(good_regex2, u'w') as f:
            f.write(json.dumps(params.as_dict()))
        params2 = Params.from_file(good_regex2)

        assert params.as_dict() == params2.as_dict()

    def test_env_var_substitution(self):
        substitutor = self.TEST_DIR / u'substitutor.jsonnet'
        key = u'TEST_ENV_VAR_SUBSTITUTION'

        assert os.environ.get(key) is None

        with open(substitutor, u'w') as f:
            f.write('{{"path": std.extVar("{key}")}}')

        # raises without environment variable set
        with pytest.raises(RuntimeError):
            Params.from_file(substitutor)

        os.environ[key] = u"PERFECT"

        params = Params.from_file(substitutor)
        assert params[u'path'] == u"PERFECT"

        del os.environ[key]

    @pytest.mark.xfail(not os.path.exists(AllenNlpTestCase.PROJECT_ROOT / u"training_config"),
                       reason=u"Training configs not installed with pip")
    def test_known_configs(self):
        configs = os.listdir(self.PROJECT_ROOT / u"training_config")

        # Our configs use environment variable substitution, and the _jsonnet parser
        # will fail if we don't pass it correct environment variables.
        forced_variables = [
            # constituency parser
            u'PTB_TRAIN_PATH', u'PTB_DEV_PATH', u'PTB_TEST_PATH',

            # srl_elmo_5.5B
            u'SRL_TRAIN_DATA_PATH', u'SRL_VALIDATION_DATA_PATH',

            # coref
            u'COREF_TRAIN_DATA_PATH', u'COREF_DEV_DATA_PATH', u'COREF_TEST_DATA_PATH',

            # ner
            u'NER_TRAIN_DATA_PATH', u'NER_TEST_A_PATH', u'NER_TEST_B_PATH'
        ]

        for var in forced_variables:
            os.environ[var] = os.environ.get(var) or unicode(self.TEST_DIR)

        for config in configs:
            try:
                Params.from_file(self.PROJECT_ROOT / u"training_config" / config)
            except Exception as e:
                raise AssertionError("unable to load params for {config}, because {e}")

        for var in forced_variables:
            if os.environ[var] == unicode(self.TEST_DIR):
                del os.environ[var]

    def test_add_file_to_archive(self):
        # Creates actual files since add_file_to_archive will throw an exception
        # if the file does not exist.
        tempdir = tempfile.mkdtemp()
        my_file = os.path.join(tempdir, u"my_file.txt")
        my_other_file = os.path.join(tempdir, u"my_other_file.txt")
        open(my_file, u'w').close()
        open(my_other_file, u'w').close()

        # Some nested classes just to exercise the ``from_params``
        # and ``add_file_to_archive`` methods.

        class C(object):
            def __init__(self, c_file     )        :
                self.c_file = c_file

            @classmethod
            def from_params(cls, params        )       :
                params.add_file_to_archive(u"c_file")
                c_file = params.pop(u"c_file")

                return cls(c_file)

        class B(object):
            def __init__(self, filename     , c)        :
                self.filename = filename
                self.c_dict = {u"here": c}

            @classmethod
            def from_params(cls, params        )       :
                params.add_file_to_archive(u"filename")

                filename = params.pop(u"filename")
                c_params = params.pop(u"c")
                c = C.from_params(c_params)

                return cls(filename, c)

        class A(object):
            def __init__(self, b)        :
                self.b = b

            @classmethod
            def from_params(cls, params        )       :
                b_params = params.pop(u"b")
                return cls(B.from_params(b_params))

        params = Params({
                u"a": {
                        u"b": {
                                u"filename": my_file,
                                u"c": {
                                        u"c_file": my_other_file
                                }
                        }
                }
        })

        # Construct ``A`` from params but then just throw it away.
        A.from_params(params.pop(u"a"))

        assert params.files_to_archive == {
                u"a.b.filename": my_file,
                u"a.b.c.c_file": my_other_file
        }

    def test_as_ordered_dict(self):
        # keyD > keyC > keyE; keyDA > keyDB; Next all other keys alphabetically
        preference_orders = [[u"keyD", u"keyC", u"keyE"], [u"keyDA", u"keyDB"]]
        params = Params({u"keyC": u"valC", u"keyB": u"valB", u"keyA": u"valA", u"keyE": u"valE",
                         u"keyD": {u"keyDB": u"valDB", u"keyDA": u"valDA"}})
        ordered_params_dict = params.as_ordered_dict(preference_orders)
        expected_ordered_params_dict = OrderedDict({u'keyD': {u'keyDA': u'valDA', u'keyDB': u'valDB'},
                                                    u'keyC': u'valC', u'keyE': u'valE',
                                                    u'keyA': u'valA', u'keyB': u'valB'})
        assert json.dumps(ordered_params_dict) == json.dumps(expected_ordered_params_dict)

    def test_to_file(self):
        # Test to_file works with or without preference orders
        params_dict = {u"keyA": u"valA", u"keyB": u"valB"}
        expected_ordered_params_dict = OrderedDict({u"keyB": u"valB", u"keyA": u"valA"})
        params = Params(params_dict)
        file_path = self.TEST_DIR / u'config.jsonnet'
        # check with preference orders
        params.to_file(file_path, [[u"keyB", u"keyA"]])
        with open(file_path, u"r") as handle:
            ordered_params_dict = OrderedDict(json.load(handle))
        assert json.dumps(expected_ordered_params_dict) == json.dumps(ordered_params_dict)
        # check without preference orders doesn't give error
        params.to_file(file_path)





from __future__ import with_statement
from __future__ import absolute_import
from builtins import str
import os
import pathlib
# from subprocess import run
run = None
#typing
import shutil

#overrides

from allennlp.common.file_utils import cached_path
from allennlp.common.util import JsonDict, sanitize
from allennlp.data import DatasetReader, Instance
from allennlp.models import Model
from allennlp.predictors.predictor import Predictor
from io import open

# TODO(mattg): We should merge how this works with how the `WikiTablesAccuracy` metric works, maybe
# just removing the need for adding this stuff at all, because the parser already runs the java
# process.  This requires modifying the scala `wikitables-executor` code to also return the
# denotation when running it as a server, and updating the model to parse the output correctly, but
# that shouldn't be too hard.
DEFAULT_EXECUTOR_JAR = u"https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-executor-0.1.0.jar"
ABBREVIATIONS_FILE = u"https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-abbreviations.tsv"
GROW_FILE = u"https://s3-us-west-2.amazonaws.com/allennlp/misc/wikitables-grow.grammar"
SEMPRE_DIR = str(pathlib.Path(u'data/'))
SEMPRE_ABBREVIATIONS_PATH = os.path.join(SEMPRE_DIR, u"abbreviations.tsv")
SEMPRE_GRAMMAR_PATH = os.path.join(SEMPRE_DIR, u"grow.grammar")

class WikiTablesParserPredictor(Predictor):
    u"""
    Wrapper for the
    :class:`~allennlp.models.encoder_decoders.wikitables_semantic_parser.WikiTablesSemanticParser`
    model.
    """

    def __init__(self, model       , dataset_reader               )        :
        super(WikiTablesParserPredictor, self).__init__(model, dataset_reader)
        # Load auxiliary sempre files during startup for faster logical form execution.
        os.makedirs(SEMPRE_DIR, exist_ok=True)
        abbreviations_path = os.path.join(SEMPRE_DIR, u'abbreviations.tsv')
        if not os.path.exists(abbreviations_path):
            run('wget {ABBREVIATIONS_FILE}', shell=True)
            run('mv wikitables-abbreviations.tsv {abbreviations_path}', shell=True)

        grammar_path = os.path.join(SEMPRE_DIR, u'grow.grammar')
        if not os.path.exists(grammar_path):
            run('wget {GROW_FILE}', shell=True)
            run('mv wikitables-grow.grammar {grammar_path}', shell=True)

    #overrides
    def _json_to_instance(self, json_dict          )            :
        u"""
        Expects JSON that looks like ``{"question": "...", "table": "..."}``.
        """
        question_text = json_dict[u"question"]
        table_rows = json_dict[u"table"].split(u'\n')

        # pylint: disable=protected-access
        tokenized_question = self._dataset_reader._tokenizer.tokenize(question_text.lower())  # type: ignore
        # pylint: enable=protected-access
        instance = self._dataset_reader.text_to_instance(question_text,  # type: ignore
                                                         table_rows,
                                                         tokenized_question=tokenized_question)
        return instance

    #overrides
    def predict_instance(self, instance          )            :
        outputs = self._model.forward_on_instance(instance)
        outputs[u'answer'] = self._execute_logical_form_on_table(outputs[u'logical_form'],
                                                                outputs[u'original_table'])
        return sanitize(outputs)


    def predict_batch_instance(self, instances                )                  :
        outputs = self._model.forward_on_instances(instances)
        for output in outputs:
            output[u'answer'] = self._execute_logical_form_on_table(output[u'logical_form'],
                                                                   output[u'original_table'])
        return sanitize(outputs)

    @staticmethod
    def _execute_logical_form_on_table(logical_form     , table     ):
        u"""
        The parameters are written out to files which the jar file reads and then executes the
        logical form.
        """
        logical_form_filename = os.path.join(SEMPRE_DIR, u'logical_forms.txt')
        with open(logical_form_filename, u'w') as temp_file:
            temp_file.write(logical_form + u'\n')

        table_dir = os.path.join(SEMPRE_DIR, u'tsv/')
        os.makedirs(table_dir, exist_ok=True)
        # The .tsv file extension is important here since the table string parameter is in tsv format.
        # If this file was named with suffix .csv then Sempre would interpret it as comma separated
        # and return the wrong denotation.
        table_filename = u'context.tsv'
        with open(os.path.join(table_dir, table_filename), u'w', encoding=u'utf-8') as temp_file:
            temp_file.write(table)

        # The id, target, and utterance are ignored, we just need to get the
        # table filename into sempre's lisp format.
        test_record = (u'(example (id nt-0) (utterance none) (context (graph tables.TableKnowledgeGraph %s))'
                       u'(targetValue (list (description "6"))))' % (table_filename))
        test_data_filename = os.path.join(SEMPRE_DIR, u'data.examples')
        with open(test_data_filename, u'w') as temp_file:
            temp_file.write(test_record)

        # TODO(matt): The jar that we have isn't optimal for this use case - we're using a
        # script designed for computing accuracy, and just pulling out a piece of it. Writing
        # a new entry point to the jar that's tailored for this use would be cleaner.
        command = u' '.join([u'java',
                            u'-jar',
                            cached_path(DEFAULT_EXECUTOR_JAR),
                            test_data_filename,
                            logical_form_filename,
                            table_dir])
        run(command, shell=True)

        denotations_file = os.path.join(SEMPRE_DIR, u'logical_forms_denotations.tsv')
        with open(denotations_file) as temp_file:
            line = temp_file.readline().split(u'\t')

        # Clean up all the temp files generated from this function.
        # Take care to not remove the auxiliary sempre files
        os.remove(logical_form_filename)
        shutil.rmtree(table_dir)
        os.remove(denotations_file)
        os.remove(test_data_filename)
        return line[1] if len(line) > 1 else line[0]

WikiTablesParserPredictor = Predictor.register(u'wikitables-parser')(WikiTablesParserPredictor)




from __future__ import absolute_import
#typing

import torch

class ChecklistState(object):
    u"""
    This class keeps track of checklist related variables that are used while training a coverage
    based semantic parser (or any other kind of transition based constrained decoder). This is
    inteded to be used within a ``DecoderState``.

    Parameters
    ----------
    terminal_actions : ``torch.Tensor``
        A vector containing the indices of terminal actions, required for computing checklists for
        next states based on current actions. The idea is that we will build checklists
        corresponding to the presence or absence of just the terminal actions. But in principle,
        they can be all actions that are relevant to checklist computation.
    checklist_target : ``torch.Tensor``
        Targets corresponding to checklist that indicate the states in which we want the checklist to
        ideally be. It is the same size as ``terminal_actions``, and it contains 1 for each corresponding
        action in the list that we want to see in the final logical form, and 0 for each corresponding
        action that we do not.
    checklist_mask : ``torch.Tensor``
        Mask corresponding to ``terminal_actions``, indicating which of those actions are relevant
        for checklist computation. For example, if the parser is penalizing non-agenda terminal
        actions, all the terminal actions are relevant.
    checklist : ``torch.Tensor``
        A checklist indicating how many times each action in its agenda has been chosen previously.
        It contains the actual counts of the agenda actions.
    terminal_indices_dict: ``Dict[int, int]``, optional
        Mapping from batch action indices to indices in any of the four vectors above. If not
        provided, this mapping will be computed here.
    """
    def __init__(self,
                 terminal_actions              ,
                 checklist_target              ,
                 checklist_mask              ,
                 checklist              ,
                 terminal_indices_dict                 = None)        :
        self.terminal_actions = terminal_actions
        self.checklist_target = checklist_target
        self.checklist_mask = checklist_mask
        self.checklist = checklist
        if terminal_indices_dict is not None:
            self.terminal_indices_dict = terminal_indices_dict
        else:
            self.terminal_indices_dict =  {}
            for checklist_index, batch_action_index in enumerate(terminal_actions.detach().cpu()):
                action_index = int(batch_action_index[0])
                if action_index == -1:
                    continue
                self.terminal_indices_dict[action_index] = checklist_index

    def update(self, action              )                    :
        u"""
        Takes an action index, updates checklist and returns an updated state.
        """
        checklist_addition = (self.terminal_actions == action).float()
        new_checklist = self.checklist + checklist_addition
        new_checklist_state = ChecklistState(terminal_actions=self.terminal_actions,
                                             checklist_target=self.checklist_target,
                                             checklist_mask=self.checklist_mask,
                                             checklist=new_checklist,
                                             terminal_indices_dict=self.terminal_indices_dict)
        return new_checklist_state

    def get_balance(self)                :
        return self.checklist_mask * (self.checklist_target - self.checklist)




from __future__ import absolute_import
#typing

#overrides

from allennlp.common.checks import ConfigurationError
from allennlp.common.util import pad_sequence_to_length
from allennlp.data.tokenizers.token import Token
from allennlp.data.token_indexers.token_indexer import TokenIndexer
from allennlp.data.vocabulary import Vocabulary


def _make_bos_eos(
        character     ,
        padding_character     ,
        beginning_of_word_character     ,
        end_of_word_character     ,
        max_word_length     
):
    char_ids = [padding_character] * max_word_length
    char_ids[0] = beginning_of_word_character
    char_ids[1] = character
    char_ids[2] = end_of_word_character
    return char_ids

class ELMoCharacterMapper(object):
    u"""
    Maps individual tokens to sequences of character ids, compatible with ELMo.
    To be consistent with previously trained models, we include it here as special of existing
    character indexers.
    """
    max_word_length = 50

    # char ids 0-255 come from utf-8 encoding bytes
    # assign 256-300 to special chars
    beginning_of_sentence_character = 256  # <begin sentence>
    end_of_sentence_character = 257  # <end sentence>
    beginning_of_word_character = 258  # <begin word>
    end_of_word_character = 259  # <end word>
    padding_character = 260 # <padding>

    beginning_of_sentence_characters = _make_bos_eos(
            beginning_of_sentence_character,
            padding_character,
            beginning_of_word_character,
            end_of_word_character,
            max_word_length
    )
    end_of_sentence_characters = _make_bos_eos(
            end_of_sentence_character,
            padding_character,
            beginning_of_word_character,
            end_of_word_character,
            max_word_length
    )

    bos_token = u'<S>'
    eos_token = u'</S>'

    @staticmethod
    def convert_word_to_char_ids(word     )             :
        if word == ELMoCharacterMapper.bos_token:
            char_ids = ELMoCharacterMapper.beginning_of_sentence_characters
        elif word == ELMoCharacterMapper.eos_token:
            char_ids = ELMoCharacterMapper.end_of_sentence_characters
        else:
            word_encoded = word.encode(u'utf-8', u'ignore')[:(ELMoCharacterMapper.max_word_length-2)]
            char_ids = [ELMoCharacterMapper.padding_character] * ELMoCharacterMapper.max_word_length
            char_ids[0] = ELMoCharacterMapper.beginning_of_word_character
            for k, chr_id in enumerate(word_encoded, start=1):
                if not(isinstance(chr_id, int)):
                    char_ids[k] = ord(chr_id)
                else:
                    char_ids[k] = chr_id
            char_ids[len(word_encoded) + 1] = ELMoCharacterMapper.end_of_word_character

        # +1 one for masking
        return [c + 1 for c in char_ids]


class ELMoTokenCharactersIndexer(TokenIndexer):
    u"""
    Convert a token to an array of character ids to compute ELMo representations.

    Parameters
    ----------
    namespace : ``str``, optional (default=``elmo_characters``)
    """
    # pylint: disable=no-self-use
    def __init__(self,
                 namespace      = u'elmo_characters')        :
        self._namespace = namespace

    #overrides
    def count_vocab_items(self, token       , counter                           ):
        pass

    #overrides
    def tokens_to_indices(self,
                          tokens             ,
                          vocabulary            ,
                          index_name     )                              :
        # pylint: disable=unused-argument
        texts = [token.text for token in tokens]

        if any(text is None for text in texts):
            raise ConfigurationError(u'ELMoTokenCharactersIndexer needs a tokenizer '
                                     u'that retains text')
        return {index_name: [ELMoCharacterMapper.convert_word_to_char_ids(text) for text in texts]}

    #overrides
    def get_padding_lengths(self, token           )                  :
        # pylint: disable=unused-argument
        return {}

    #overrides
    def get_padding_token(self)             :
        return []

    @staticmethod
    def _default_value_for_padding():
        return [0] * ELMoCharacterMapper.max_word_length

    #overrides
    def pad_token_sequence(self,
                           tokens                            ,
                           desired_num_tokens                ,
                           padding_lengths                )                              :
        # pylint: disable=unused-argument
        return dict((key, pad_sequence_to_length(val, desired_num_tokens[key],
                                            default_value=self._default_value_for_padding))
                for key, val in list(tokens.items()))

ELMoTokenCharactersIndexer = TokenIndexer.register(u"elmo_characters")(ELMoTokenCharactersIndexer)




from __future__ import absolute_import
from collections import defaultdict
#typing

from allennlp.common.util import START_SYMBOL
from allennlp.semparse.worlds.world import World
from allennlp.semparse.type_declarations import type_declaration as types


class ActionSpaceWalker(object):
    u"""
    ``ActionSpaceWalker`` takes a world, traverses all the valid paths driven by the valid action
    specification of the world to generate all possible logical forms (under some constraints). This
    class also has some utilities for indexing logical forms to efficiently retrieve required
    subsets.

    Parameters
    ----------
    world : ``World``
        The world from which valid actions will be taken.
    max_path_length : ``int``
        The maximum path length till which the action space will be explored. Paths longer than this
        length will be discarded.
    """
    def __init__(self, world       , max_path_length     )        :
        self._world = world
        self._max_path_length = max_path_length
        self._completed_paths =  None
        self._terminal_path_index =  defaultdict(set)
        self._length_sorted_paths =  None

    def _walk(self)        :
        u"""
        Walk over action space to collect completed paths of at most ``self._max_path_length`` steps.
        """
        # Buffer of NTs to expand, previous actions
        incomplete_paths = [([unicode(type_)], ["{START_SYMBOL} -> {type_}"]) for type_ in
                            self._world.get_valid_starting_types()]

        self._completed_paths = []
        actions = self._world.get_valid_actions()
        # Overview: We keep track of the buffer of non-terminals to expand, and the action history
        # for each incomplete path. At every iteration in the while loop below, we iterate over all
        # incomplete paths, expand one non-terminal from the buffer in a depth-first fashion, get
        # all possible next actions triggered by that non-terminal and add to the paths. Then, we
        # check the expanded paths, to see if they are 1) complete, in which case they are
        # added to completed_paths, 2) longer than max_path_length, in which case they are
        # discarded, or 3) neither, in which case they are used to form the incomplete_paths for the
        # next iteration of this while loop.
        # While the non-terminal expansion is done in a depth-first fashion, note that the search over
        # the action space itself is breadth-first.
        while incomplete_paths:
            next_paths = []
            for nonterminal_buffer, history in incomplete_paths:
                # Taking the last non-terminal added to the buffer. We're going depth-first.
                nonterminal = nonterminal_buffer.pop()
                # Iterating over all possible next actions.
                for action in actions[nonterminal]:
                    new_history = history + [action]
                    new_nonterminal_buffer = nonterminal_buffer[:]
                    # Since we expand the last action added to the buffer, the left child should be
                    # added after the right child.
                    for right_side_part in reversed(self._get_right_side_parts(action)):
                        if types.is_nonterminal(right_side_part):
                            new_nonterminal_buffer.append(right_side_part)
                    next_paths.append((new_nonterminal_buffer, new_history))
            incomplete_paths = []
            for nonterminal_buffer, path in next_paths:
                # An empty buffer means that we've completed this path.
                if not nonterminal_buffer:
                    # Indexing completed paths by the nonterminals they contain.
                    next_path_index = len(self._completed_paths)
                    for action in path:
                        for value in self._get_right_side_parts(action):
                            if not types.is_nonterminal(value):
                                self._terminal_path_index[action].add(next_path_index)
                    self._completed_paths.append(path)
                # We're adding to incomplete_paths for the next iteration, only those paths that are
                # shorter than the max_path_length. The remaining paths will be discarded.
                elif len(path) <= self._max_path_length:
                    incomplete_paths.append((nonterminal_buffer, path))

    @staticmethod
    def _get_right_side_parts(action     )             :
        _, right_side = action.split(u" -> ")
        if u"[" in right_side:
            right_side_parts = right_side[1:-1].split(u", ")
        else:
            right_side_parts = [right_side]
        return right_side_parts

    def get_logical_forms_with_agenda(self,
                                      agenda           ,
                                      max_num_logical_forms      = None)             :
        if self._completed_paths is None:
            self._walk()
        agenda_path_indices = [self._terminal_path_index[action] for action in agenda]
        # TODO (pradeep): Sort the indices and do intersections in order, so that we can return the
        # set with maximal coverage if the full intersection is null.
        return_set = agenda_path_indices[0]
        for next_set in agenda_path_indices[1:]:
            return_set = return_set.intersection(next_set)
        paths = [self._completed_paths[index] for index in return_set]
        if max_num_logical_forms is not None:
            paths = sorted(paths, key=len)[:max_num_logical_forms]
        logical_forms = [self._world.get_logical_form(path) for path in paths]
        return logical_forms

    def get_all_logical_forms(self,
                              max_num_logical_forms      = None)             :
        if self._completed_paths is None:
            self._walk()
        paths = self._completed_paths
        if max_num_logical_forms is not None:
            if self._length_sorted_paths is None:
                self._length_sorted_paths = sorted(self._completed_paths, key=len)
            paths = self._length_sorted_paths[:max_num_logical_forms]
        logical_forms = [self._world.get_logical_form(path) for path in paths]
        return logical_forms




from __future__ import absolute_import
import logging
#typing

#overrides

import torch

from allennlp.data.fields.production_rule_field import ProductionRuleArray
from allennlp.data.vocabulary import Vocabulary
from allennlp.models.model import Model
from allennlp.modules import TextFieldEmbedder, Seq2SeqEncoder, Embedding
from allennlp.nn import util
from allennlp.nn.decoding import GrammarState, RnnState
from allennlp.semparse.type_declarations import type_declaration
from allennlp.semparse.type_declarations.type_declaration import START_SYMBOL
from allennlp.semparse.worlds import NlvrWorld
from allennlp.training.metrics import Average
try:
    from itertools import izip
except:
    izip = zip


logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class NlvrSemanticParser(Model):
    u"""
    ``NlvrSemanticParser`` is a semantic parsing model built for the NLVR domain. This is an
    abstract class and does not have a ``forward`` method implemented. Classes that inherit from
    this class are expected to define their own logic depending on the kind of supervision they use.
    Accordingly, they should use the appropriate ``DecoderTrainer``. This class provides some common
    functionality for things like defining an initial ``RnnState``, embedding actions, evaluating
    the denotations of completed logical forms, etc.  There is a lot of overlap with
    ``WikiTablesSemanticParser`` here. We may want to eventually move the common functionality into
    a more general transition-based parsing class.

    Parameters
    ----------
    vocab : ``Vocabulary``
    sentence_embedder : ``TextFieldEmbedder``
        Embedder for sentences.
    action_embedding_dim : ``int``
        Dimension to use for action embeddings.
    encoder : ``Seq2SeqEncoder``
        The encoder to use for the input question.
    dropout : ``float``, optional (default=0.0)
        Dropout on the encoder outputs.
    rule_namespace : ``str``, optional (default=rule_labels)
        The vocabulary namespace to use for production rules.  The default corresponds to the
        default used in the dataset reader, so you likely don't need to modify this.
    """
    def __init__(self,
                 vocab            ,
                 sentence_embedder                   ,
                 action_embedding_dim     ,
                 encoder                ,
                 dropout        = 0.0,
                 rule_namespace      = u'rule_labels')        :
        super(NlvrSemanticParser, self).__init__(vocab=vocab)

        self._sentence_embedder = sentence_embedder
        self._denotation_accuracy = Average()
        self._consistency = Average()
        self._encoder = encoder
        if dropout > 0:
            self._dropout = torch.nn.Dropout(p=dropout)
        else:
            self._dropout = lambda x: x
        self._rule_namespace = rule_namespace

        self._action_embedder = Embedding(num_embeddings=vocab.get_vocab_size(self._rule_namespace),
                                          embedding_dim=action_embedding_dim)

        # This is what we pass as input in the first step of decoding, when we don't have a
        # previous action.
        self._first_action_embedding = torch.nn.Parameter(torch.FloatTensor(action_embedding_dim))
        torch.nn.init.normal_(self._first_action_embedding)

    #overrides
    def forward(self):  # type: ignore
        # pylint: disable=arguments-differ
        # Sub-classes should define their own logic here.
        raise NotImplementedError

    def _get_initial_rnn_state(self, sentence                             ):
        embedded_input = self._sentence_embedder(sentence)
        # (batch_size, sentence_length)
        sentence_mask = util.get_text_field_mask(sentence).float()

        batch_size = embedded_input.size(0)

        # (batch_size, sentence_length, encoder_output_dim)
        encoder_outputs = self._dropout(self._encoder(embedded_input, sentence_mask))

        final_encoder_output = util.get_final_encoder_states(encoder_outputs,
                                                             sentence_mask,
                                                             self._encoder.is_bidirectional())
        memory_cell = encoder_outputs.new_zeros(batch_size, self._encoder.get_output_dim())
        attended_sentence = self._decoder_step.attend_on_sentence(final_encoder_output,
                                                                  encoder_outputs, sentence_mask)
        encoder_outputs_list = [encoder_outputs[i] for i in range(batch_size)]
        sentence_mask_list = [sentence_mask[i] for i in range(batch_size)]
        initial_rnn_state = []
        for i in range(batch_size):
            initial_rnn_state.append(RnnState(final_encoder_output[i],
                                              memory_cell[i],
                                              self._first_action_embedding,
                                              attended_sentence[i],
                                              encoder_outputs_list,
                                              sentence_mask_list))
        return initial_rnn_state

    def _get_label_strings(self, labels):
        # TODO (pradeep): Use an unindexed field for labels?
        labels_data = labels.detach().cpu()
        label_strings                  = []
        for instance_labels_data in labels_data:
            label_strings.append([])
            for label in instance_labels_data:
                label_int = int(label)
                if label_int == -1:
                    # Padding, because not all instances have the same number of labels.
                    continue
                label_strings[-1].append(self.vocab.get_token_from_index(label_int, u"denotations"))
        return label_strings

    @classmethod
    def _get_action_strings(cls,
                            possible_actions                                 ,
                            action_indices                            )                         :
        u"""
        Takes a list of possible actions and indices of decoded actions into those possible actions
        for a batch and returns sequences of action strings. We assume ``action_indices`` is a dict
        mapping batch indices to k-best decoded sequence lists.
        """
        all_action_strings                        = []
        batch_size = len(possible_actions)
        for i in range(batch_size):
            batch_actions = possible_actions[i]
            batch_best_sequences = action_indices[i] if i in action_indices else []
            # This will append an empty list to ``all_action_strings`` if ``batch_best_sequences``
            # is empty.
            action_strings = [[batch_actions[rule_id][0] for rule_id in sequence]
                              for sequence in batch_best_sequences]
            all_action_strings.append(action_strings)
        return all_action_strings

    @staticmethod
    def _get_denotations(action_strings                       ,
                         worlds                       )                         :
        all_denotations                        = []
        for instance_worlds, instance_action_sequences in izip(worlds, action_strings):
            denotations                  = []
            for instance_action_strings in instance_action_sequences:
                if not instance_action_strings:
                    continue
                logical_form = instance_worlds[0].get_logical_form(instance_action_strings)
                instance_denotations            = []
                for world in instance_worlds:
                    # Some of the worlds can be None for instances that come with less than 4 worlds
                    # because of padding.
                    if world is not None:
                        instance_denotations.append(unicode(world.execute(logical_form)))
                denotations.append(instance_denotations)
            all_denotations.append(denotations)
        return all_denotations

    @staticmethod
    def _check_denotation(best_action_sequence           ,
                          labels           ,
                          worlds                 )              :
        is_correct = []
        for world, label in izip(worlds, labels):
            logical_form = world.get_logical_form(best_action_sequence)
            denotation = world.execute(logical_form)
            is_correct.append(unicode(denotation).lower() == label)
        return is_correct

    @staticmethod
    def _create_grammar_state(world           ,
                              possible_actions                           )                :
        valid_actions = world.get_valid_actions()
        action_mapping = {}
        for i, action in enumerate(possible_actions):
            action_mapping[action[0]] = i
        translated_valid_actions = {}
        for key, action_strings in list(valid_actions.items()):
            translated_valid_actions[key] = [action_mapping[action_string]
                                             for action_string in action_strings]
        return GrammarState([START_SYMBOL],
                            {},
                            translated_valid_actions,
                            action_mapping,
                            type_declaration.is_nonterminal)

    def _embed_actions(self, actions                                 ):                       
                                                                                                           
        u"""
        Given all of the possible actions for all batch instances, produce an embedding for them.
        There will be significant overlap in this list, as the production rules from the grammar
        are shared across all batch instances.  Our returned tensor has an embedding for each
        `unique` action, so we also need to return a mapping from the original ``(batch_index,
        action_index)`` to our new ``global_action_index``, so that we can get the right action
        embedding during decoding.

        Returns
        -------
        action_embeddings : ``torch.Tensor``
            Has shape ``(num_unique_actions, action_embedding_dim)``.
        action_map : ``Dict[Tuple[int, int], int]``
            Maps ``(batch_index, action_index)`` in the input action list to ``action_index`` in
            the ``action_embeddings`` tensor.
        """
        # TODO(mattg): This whole action pipeline might be a whole lot more complicated than it
        # needs to be.  We used to embed actions differently (using some crazy ideas about
        # embedding the LHS and RHS separately); we could probably get away with simplifying things
        # further now that we're just doing a simple embedding for global actions.  But I'm leaving
        # it like this for now to have a minimal change to go from the LHS/RHS embedding to a
        # single action embedding.
        embedded_actions = self._action_embedder.weight

        # Now we just need to make a map from `(batch_index, action_index)` to
        # `global_action_index`.  global_action_ids has the list of all unique actions; here we're
        # going over all of the actions for each batch instance so we can map them to the global
        # action ids.
        action_vocab = self.vocab.get_token_to_index_vocabulary(self._rule_namespace)
        action_map                             = {}
        for batch_index, instance_actions in enumerate(actions):
            for action_index, action in enumerate(instance_actions):
                if not action[0]:
                    # This rule is padding.
                    continue
                global_action_id = action_vocab[action[0]]
                action_map[(batch_index, action_index)] = global_action_id
        return embedded_actions, action_map

    #overrides
    def decode(self, output_dict                         )                           :
        u"""
        This method overrides ``Model.decode``, which gets called after ``Model.forward``, at test
        time, to finalize predictions. We only transform the action string sequences into logical
        forms here.
        """
        best_action_strings = output_dict[u"best_action_strings"]
        # Instantiating an empty world for getting logical forms.
        world = NlvrWorld([])
        logical_forms = []
        for instance_action_sequences in best_action_strings:
            instance_logical_forms = []
            for action_strings in instance_action_sequences:
                if action_strings:
                    instance_logical_forms.append(world.get_logical_form(action_strings))
                else:
                    instance_logical_forms.append(u'')
            logical_forms.append(instance_logical_forms)
        output_dict[u"logical_form"] = logical_forms
        return output_dict

    def _check_state_denotations(self, state)              :
        u"""
        Returns whether action history in the state evaluates to the correct denotations over all
        worlds. Only defined when the state is finished.
        """
        assert state.is_finished(), u"Cannot compute denotations for unfinished states!"
        # Since this is a finished state, its group size must be 1.
        batch_index = state.batch_indices[0]
        worlds = state.worlds[batch_index]
        instance_label_strings = state.label_strings[batch_index]
        history = state.action_history[0]
        all_actions = state.possible_actions[0]
        action_sequence = [all_actions[action][0] for action in history]
        return self._check_denotation(action_sequence, instance_label_strings, worlds)



# pylint: disable=no-self-use,invalid-name


from __future__ import division
from __future__ import absolute_import
from pytest import approx

from allennlp.common.testing import AllenNlpTestCase
from allennlp.models.archival import load_archive
from allennlp.predictors import Predictor


class TestBidafPredictor(AllenNlpTestCase):
    def test_uses_named_inputs(self):
        inputs = {
                u"question": u"What kind of test succeeded on its first attempt?",
                u"passage": u"One time I was writing a unit test, and it succeeded on the first attempt."
        }

        archive = load_archive(self.FIXTURES_ROOT / u'bidaf' / u'serialization' / u'model.tar.gz')
        predictor = Predictor.from_archive(archive, u'machine-comprehension')

        result = predictor.predict_json(inputs)

        best_span = result.get(u"best_span")
        assert best_span is not None
        assert isinstance(best_span, list)
        assert len(best_span) == 2
        assert all(isinstance(x, int) for x in best_span)
        assert best_span[0] <= best_span[1]

        best_span_str = result.get(u"best_span_str")
        assert isinstance(best_span_str, unicode)
        assert best_span_str != u""

        for probs_key in (u"span_start_probs", u"span_end_probs"):
            probs = result.get(probs_key)
            assert probs is not None
            assert all(isinstance(x, float) for x in probs)
            assert sum(probs) == approx(1.0)

    def test_batch_prediction(self):
        inputs = [
                {
                        u"question": u"What kind of test succeeded on its first attempt?",
                        u"passage": u"One time I was writing a unit test, and it succeeded on the first attempt."
                },
                {
                        u"question": u"What kind of test succeeded on its first attempt at batch processing?",
                        u"passage": u"One time I was writing a unit test, and it always failed!"
                }
        ]

        archive = load_archive(self.FIXTURES_ROOT / u'bidaf' / u'serialization' / u'model.tar.gz')
        predictor = Predictor.from_archive(archive, u'machine-comprehension')

        results = predictor.predict_batch_json(inputs)
        assert len(results) == 2

        for result in results:
            best_span = result.get(u"best_span")
            best_span_str = result.get(u"best_span_str")
            start_probs = result.get(u"span_start_probs")
            end_probs = result.get(u"span_end_probs")
            assert best_span is not None
            assert isinstance(best_span, list)
            assert len(best_span) == 2
            assert all(isinstance(x, int) for x in best_span)
            assert best_span[0] <= best_span[1]

            assert isinstance(best_span_str, unicode)
            assert best_span_str != u""

            for probs in (start_probs, end_probs):
                assert probs is not None
                assert all(isinstance(x, float) for x in probs)
                assert sum(probs) == approx(1.0)




from __future__ import absolute_import
#overrides
import torch
from torch.nn.parameter import Parameter

from allennlp.modules.similarity_functions.similarity_function import SimilarityFunction
from allennlp.nn import Activation


class BilinearSimilarity(SimilarityFunction):
    u"""
    This similarity function performs a bilinear transformation of the two input vectors.  This
    function has a matrix of weights ``W`` and a bias ``b``, and the similarity between two vectors
    ``x`` and ``y`` is computed as ``x^T W y + b``.

    Parameters
    ----------
    tensor_1_dim : ``int``
        The dimension of the first tensor, ``x``, described above.  This is ``x.size()[-1]`` - the
        length of the vector that will go into the similarity computation.  We need this so we can
        build the weight matrix correctly.
    tensor_2_dim : ``int``
        The dimension of the second tensor, ``y``, described above.  This is ``y.size()[-1]`` - the
        length of the vector that will go into the similarity computation.  We need this so we can
        build the weight matrix correctly.
    activation : ``Activation``, optional (default=linear (i.e. no activation))
        An activation function applied after the ``x^T W y + b`` calculation.  Default is no
        activation.
    """
    def __init__(self,
                 tensor_1_dim     ,
                 tensor_2_dim     ,
                 activation             = None)        :
        super(BilinearSimilarity, self).__init__()
        self._weight_matrix = Parameter(torch.Tensor(tensor_1_dim, tensor_2_dim))
        self._bias = Parameter(torch.Tensor(1))
        self._activation = activation or Activation.by_name(u'linear')()
        self.reset_parameters()

    def reset_parameters(self):
        torch.nn.init.xavier_uniform_(self._weight_matrix)
        self._bias.data.fill_(0)

    #overrides
    def forward(self, tensor_1              , tensor_2              )                :
        intermediate = torch.matmul(tensor_1, self._weight_matrix)
        result = (intermediate * tensor_2).sum(dim=-1)
        return self._activation(result + self._bias)

BilinearSimilarity = SimilarityFunction.register(u"bilinear")(BilinearSimilarity)



u"""
The ``test-install`` subcommand verifies
an installation by running the unit tests.

.. code-block:: bash

    $ allennlp test-install --help
    usage: allennlp test-install [-h] [--run-all]
                                 [--include-package INCLUDE_PACKAGE]

    Test that installation works by running the unit tests.

    optional arguments:
      -h, --help            show this help message and exit
      --run-all             By default, we skip tests that are slow or download
                            large files. This flag will run all tests.
      --include-package INCLUDE_PACKAGE
                            additional packages to include
"""


from __future__ import absolute_import
import argparse
import logging
import os
import pathlib

import pytest

import allennlp
from allennlp.commands.subcommand import Subcommand

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

class TestInstall(Subcommand):
    def add_subparser(self, name     , parser                            )                           :
        # pylint: disable=protected-access
        description = u'''Test that installation works by running the unit tests.'''
        subparser = parser.add_parser(
                name, description=description, help=u'Run the unit tests.')

        subparser.add_argument(u'--run-all', action=u"store_true",
                               help=u"By default, we skip tests that are slow "
                               u"or download large files. This flag will run all tests.")

        subparser.set_defaults(func=_run_test)

        return subparser


def _get_module_root():
    return pathlib.Path(allennlp.__file__).parent


def _run_test(args                    ):
    initial_working_dir = os.getcwdu()
    module_parent = _get_module_root().parent
    logger.info(u"Changing directory to %s", module_parent)
    os.chdir(module_parent)
    test_dir = os.path.join(module_parent, u"allennlp")
    logger.info(u"Running tests at %s", test_dir)
    if args.run_all:
        # TODO(nfliu): remove this when notebooks have been rewritten as markdown.
        exit_code = pytest.main([test_dir, u'--color=no', u'-k', u'not notebooks_test'])
    else:
        exit_code = pytest.main([test_dir, u'--color=no', u'-k', u'not sniff_test and not notebooks_test',
                                 u'-m', u'not java'])
    # Change back to original working directory after running tests
    os.chdir(initial_working_dir)
    exit(exit_code)
